#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_run_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[28]*x[0] + 314.15926535897933*y[44] - 314.15926535897933*y[48]; 
data[1] = -x[2] + y[45]; 
data[2] = (-x[2] + y[36]/p[12])/p[19]; 
data[3] = (u[12] - x[3])/p[20]; 
data[4] = (-x[4] + u[18])/p[21]; 
data[5] = (-x[5] + u[19])/p[21]; 
data[6] = (-x[6] + u[20])/p[21]; 
data[7] = (-x[7] + u[21])/p[21]; 
data[8] = (2*p[32]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
))))/p[37] - x[8])/p[38]; 
data[9] = (-x[9] + (-2*p[32]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
)))) + x[8])/p[33])/p[39]; 
data[10] = (-x[10] + 2*p[32]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
))))/p[34])/p[40]; 
data[11] = (-x[11] + (-p[32]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
)))) + x[8]/p[36])/p[35])/p[41]; 
data[12] = -9.9999999999999995e-7*x[12] + u[25]; 
data[13] = -p[47]*x[13] - y[48] + u[26] + 1; 

}

void g_run_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14] - y[24]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14] - y[31]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14] - y[25]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14] - y[32]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14] - y[26]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14] - y[33]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14] - y[27]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14] - y[34]; 
data[8] = -94.615102344455565*y[1] - 184.75059629806532*y[0] + 17.974903844364775*y[3] + 69.368640085185334*y[2] + 40.74868784166955*y[5] + 43.675799641021136*y[4] + 17.974903844364817*y[7] + 69.368640085185348*y[6] + 94.615102344455565*y[9] + 184.75059629806532*y[8] - 17.974903844364775*y[11] - 69.368640085185334*y[10] - 40.74868784166955*y[13] - 43.675799641021136*y[12] - 17.974903844364817*y[15] - 69.368640085185348*y[14] - y[16]; 
data[9] = -184.75059629806532*y[1] + 94.615102344455565*y[0] + 69.368640085185334*y[3] - 17.974903844364775*y[2] + 43.675799641021136*y[5] - 40.74868784166955*y[4] + 69.368640085185348*y[7] - 17.974903844364817*y[6] + 184.75059629806532*y[9] - 94.615102344455565*y[8] - 69.368640085185334*y[11] + 17.974903844364775*y[10] - 43.675799641021136*y[13] + 40.74868784166955*y[12] - 69.368640085185348*y[15] + 17.974903844364817*y[14] - y[20]; 
data[10] = 17.974903844364796*y[1] + 69.368640085185319*y[0] - 94.615102344455607*y[3] - 184.75059629806523*y[2] + 17.974903844364814*y[5] + 69.368640085185291*y[4] + 40.748687841669586*y[7] + 43.675799641021079*y[6] - 17.974903844364796*y[9] - 69.368640085185319*y[8] + 94.615102344455607*y[11] + 184.75059629806523*y[10] - 17.974903844364814*y[13] - 69.368640085185291*y[12] - 40.748687841669586*y[15] - 43.675799641021079*y[14] - y[17]; 
data[11] = 69.368640085185319*y[1] - 17.974903844364796*y[0] - 184.75059629806523*y[3] + 94.615102344455607*y[2] + 69.368640085185291*y[5] - 17.974903844364814*y[4] + 43.675799641021079*y[7] - 40.748687841669586*y[6] - 69.368640085185319*y[9] + 17.974903844364796*y[8] + 184.75059629806523*y[11] - 94.615102344455607*y[10] - 69.368640085185291*y[13] + 17.974903844364814*y[12] - 43.675799641021079*y[15] + 40.748687841669586*y[14] - y[21]; 
data[12] = 40.748687841669557*y[1] + 43.675799641021122*y[0] + 17.97490384436481*y[3] + 69.368640085185277*y[2] - 94.615102344455636*y[5] - 184.75059629806526*y[4] + 17.974903844364817*y[7] + 69.368640085185277*y[6] - 40.748687841669557*y[9] - 43.675799641021122*y[8] - 17.97490384436481*y[11] - 69.368640085185277*y[10] + 94.615102344455636*y[13] + 184.75059629806526*y[12] - 17.974903844364817*y[15] - 69.368640085185277*y[14] - y[18]; 
data[13] = 43.675799641021122*y[1] - 40.748687841669557*y[0] + 69.368640085185277*y[3] - 17.97490384436481*y[2] - 184.75059629806526*y[5] + 94.615102344455636*y[4] + 69.368640085185277*y[7] - 17.974903844364817*y[6] - 43.675799641021122*y[9] + 40.748687841669557*y[8] - 69.368640085185277*y[11] + 17.97490384436481*y[10] + 184.75059629806526*y[13] - 94.615102344455636*y[12] - 69.368640085185277*y[15] + 17.974903844364817*y[14] - y[22]; 
data[14] = 17.9749038443648*y[1] + 69.368640085185334*y[0] + 40.748687841669586*y[3] + 43.675799641021086*y[2] + 17.974903844364821*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.75059629806523*y[6] - 17.9749038443648*y[9] - 69.368640085185334*y[8] - 40.748687841669586*y[11] - 43.675799641021086*y[10] - 17.974903844364821*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.75059629806523*y[14] - y[19]; 
data[15] = 69.368640085185334*y[1] - 17.9749038443648*y[0] + 43.675799641021086*y[3] - 40.748687841669586*y[2] + 69.368640085185291*y[5] - 17.974903844364821*y[4] - 184.75059629806523*y[7] + 94.61510234445565*y[6] - 69.368640085185334*y[9] + 17.9749038443648*y[8] - 43.675799641021086*y[11] + 40.748687841669586*y[10] - 69.368640085185291*y[13] + 17.974903844364821*y[12] + 184.75059629806523*y[15] - 94.61510234445565*y[14] - y[23]; 
data[16] = -p[1]*y[16] - y[8] + 1.0*p[0]*y[20] + u[0]*cos(u[4]); 
data[17] = -p[1]*y[17] - y[10] + 1.0*p[0]*y[21] + u[1]*cos(u[4] - 2.0943951023931953); 
data[18] = -p[1]*y[18] - y[12] + 1.0*p[0]*y[22] + u[2]*cos(u[4] - 4.1887902047863905); 
data[19] = -p[3]*y[19] - y[14] + 1.0*p[2]*y[23]; 
data[20] = -1.0*p[1]*y[20] - 1.0*y[9] - 1.0*p[0]*y[16] + 1.0*u[0]*sin(u[4]); 
data[21] = -1.0*p[1]*y[21] - 1.0*y[11] - 1.0*p[0]*y[17] + 1.0*u[1]*sin(u[4] - 2.0943951023931953); 
data[22] = -1.0*p[1]*y[22] - 1.0*y[13] - 1.0*p[0]*y[18] + 1.0*u[2]*sin(u[4] - 4.1887902047863905); 
data[23] = -1.0*p[3]*y[23] - 1.0*y[15] - 1.0*p[2]*y[19]; 
data[24] = -p[7]*y[24] - y[0] + 1.0*p[6]*y[31] + 0.40824829046386307*y[41]*y[46]*cos(u[6] + y[38]) + y[28]; 
data[25] = -p[7]*y[25] - y[2] + 1.0*p[6]*y[32] + 0.40824829046386307*y[42]*y[46]*cos(u[6] + y[39]) + y[28]; 
data[26] = -p[7]*y[26] - y[4] + 1.0*p[6]*y[33] + 0.40824829046386307*y[43]*y[46]*cos(u[6] + y[40]) + y[28]; 
data[27] = -p[9]*y[27] - y[6] + 1.0*p[8]*y[34] + 0.40824829046386307*u[5]*y[46]*cos(u[7]) + y[28]; 
data[28] = p[11]*y[28]/(pow(p[11], 2) + 1.0*pow(p[10], 2)) + 1.0*p[10]*y[35]/(pow(p[11], 2) + 1.0*pow(p[10], 2)) + y[24] + y[25] + y[26] + y[27]; 
data[29] = -1.0*p[7]*y[31] - 1.0*y[1] - 1.0*p[6]*y[24] + 0.40824829046386307*y[41]*y[46]*sin(u[6] + y[38]) + 1.0*y[35]; 
data[30] = -1.0*p[7]*y[32] - 1.0*y[3] - 1.0*p[6]*y[25] + 0.40824829046386307*y[42]*y[46]*sin(u[6] + y[39]) + 1.0*y[35]; 
data[31] = -1.0*p[7]*y[33] - 1.0*y[5] - 1.0*p[6]*y[26] + 0.40824829046386307*y[43]*y[46]*sin(u[6] + y[40]) + 1.0*y[35]; 
data[32] = -1.0*p[9]*y[34] - 1.0*y[7] - 1.0*p[8]*y[27] + 0.40824829046386307*u[5]*y[46]*sin(u[7]) + 1.0*y[35]; 
data[33] = 1.0*p[11]*y[35]/(pow(p[11], 2) + 1.0*pow(p[10], 2)) - 1.0*p[10]*y[28]/(pow(p[11], 2) + 1.0*pow(p[10], 2)) + 1.0*y[31] + 1.0*y[32] + 1.0*y[33] + 1.0*y[34]; 
data[34] = p[13]*(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + p[13]*(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + p[13]*(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + p[13]*(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7) + p[14]*sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + p[14]*sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + p[14]*sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + p[14]*sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7) + 4*p[15] + 0.40824829046386307*y[31]*y[41]*y[46]*sin(u[6] + y[38]) + 0.40824829046386307*y[24]*y[41]*y[46]*cos(u[6] + y[38]) + 0.40824829046386307*y[32]*y[42]*y[46]*sin(u[6] + y[39]) + 0.40824829046386307*y[25]*y[42]*y[46]*cos(u[6] + y[39]) + 0.40824829046386307*y[33]*y[43]*y[46]*sin(u[6] + y[40]) + 0.40824829046386307*y[33]*u[5]*y[46]*sin(u[7]) + 0.40824829046386307*y[26]*y[43]*y[46]*cos(u[6] + y[40]) + 0.40824829046386307*y[26]*u[5]*y[46]*cos(u[7]) - y[29]; 
data[35] = -y[30] + y[29]/y[46]; 
data[36] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) - y[1]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[0]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] + 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] - 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] - 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] + 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[31]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[24]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[24]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) - y[36]; 
data[37] = 0.33333333333333331*y[1]*y[24] + y[1]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[1]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) - 0.33333333333333331*y[0]*y[31] + y[0]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) + y[0]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333326*y[3]*y[25] - 0.28867513459481281*y[3]*y[33] - 0.16666666666666652*y[3]*y[26] - 0.33333333333333326*y[2]*y[32] + 0.16666666666666652*y[2]*y[33] - 0.28867513459481281*y[2]*y[26] + 0.28867513459481281*y[5]*y[32] - 0.16666666666666652*y[5]*y[25] + 0.33333333333333326*y[5]*y[26] + 0.16666666666666652*y[4]*y[32] + 0.28867513459481281*y[4]*y[25] - 0.33333333333333326*y[4]*y[33] - 1.0*y[31]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[31]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[24]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[24]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) - y[37]; 
data[38] = -y[38] + atan2(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), -p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0])); 
data[39] = -y[39] + atan2(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), -p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953)); 
data[40] = -y[40] + atan2(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), -p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905)); 
data[41] = -y[41] + 3.4641016151377544*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))/y[46]; 
data[42] = -y[42] + 3.4641016151377544*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))/y[46]; 
data[43] = -y[43] + 3.4641016151377544*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))/y[46]; 
data[44] = p[23]*(-x[2] + y[45] + x[1]/p[24]) - y[44] + 1; 
data[45] = p[27]*x[13] + x[3] - y[45] - (y[44] - u[11])/p[22]; 
data[46] = -y[46] + u[22]; 
data[47] = p[43]*p[42] - p[42]*p[44]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
)))) + 4.308339024928669e-5*p[42]*u[23]*log(x[9]*sqrt(x[11])/x[10]) - y[47]; 
data[48] = -y[48] + (p[12]*y[44] + 1000.0)/(p[12] + 1000.0); 
data[49] = p[46]*x[13] + p[45]*(1 - y[48]) - y[49]; 

}

void h_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14]; 
data[8] = sqrt(1.0*pow(y[20], 2) + pow(y[16], 2)); 
data[9] = sqrt(1.0*pow(y[21], 2) + pow(y[17], 2)); 
data[10] = sqrt(1.0*pow(y[22], 2) + pow(y[18], 2)); 
data[11] = sqrt(1.0*pow(y[23], 2) + pow(y[19], 2)); 
data[12] = 1.0*y[9]*y[20] + y[8]*y[16] + 1.0*y[11]*y[21] + y[10]*y[17] + 1.0*y[13]*y[22] + y[12]*y[18]; 
data[13] = 1.0*y[9]*y[16] - 1.0*y[8]*y[20] + 1.0*y[11]*y[17] - 1.0*y[10]*y[21] + 1.0*y[13]*y[18] - 1.0*y[12]*y[22]; 
data[14] = sqrt(1.0*pow(y[31], 2) + pow(y[24], 2)); 
data[15] = sqrt(1.0*pow(y[32], 2) + pow(y[25], 2)); 
data[16] = sqrt(1.0*pow(y[33], 2) + pow(y[26], 2)); 
data[17] = sqrt(1.0*pow(y[34], 2) + pow(y[27], 2)); 
data[18] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) - y[1]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[0]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] + 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] - 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] - 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] + 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[31]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[24]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[24]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]); 
data[19] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666677*y[32] + 0.28867513459481275*y[25]) - y[1]*(0.16666666666666657*y[33] - 0.28867513459481287*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(0.28867513459481275*y[32] - 0.16666666666666677*y[25]) + y[0]*(-0.28867513459481287*y[33] - 0.16666666666666657*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] - 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] + 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] + 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] - 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666677*y[3] - 0.28867513459481275*y[2]) + 1.0*y[31]*(-0.16666666666666657*y[5] + 0.28867513459481287*y[4]) + 1.0*y[24]*(0.28867513459481275*y[3] - 0.16666666666666677*y[2]) + 1.0*y[24]*(-0.28867513459481287*y[5] - 0.16666666666666657*y[4]); 
data[20] = 0.33333333333333331*y[1]*y[31] + 0.33333333333333331*y[1]*y[32] + 0.33333333333333331*y[1]*y[33] + 0.33333333333333331*y[0]*y[24] + 0.33333333333333331*y[0]*y[25] + 0.33333333333333331*y[0]*y[26] + 0.33333333333333331*y[3]*y[31] + 0.33333333333333331*y[3]*y[32] + 0.33333333333333331*y[3]*y[33] + 0.33333333333333331*y[2]*y[24] + 0.33333333333333331*y[2]*y[25] + 0.33333333333333331*y[2]*y[26] + 0.33333333333333331*y[5]*y[31] + 0.33333333333333331*y[5]*y[32] + 0.33333333333333331*y[5]*y[33] + 0.33333333333333331*y[4]*y[24] + 0.33333333333333331*y[4]*y[25] + 0.33333333333333331*y[4]*y[26]; 
data[21] = y[41]; 
data[22] = y[42]; 
data[23] = y[43]; 
data[24] = u[5]; 
data[25] = u[6]; 
data[26] = y[29]; 
data[27] = 1.0*y[1]*y[31] + y[0]*y[24] + 1.0*y[3]*y[32] + y[2]*y[25] + 1.0*y[5]*y[33] + y[4]*y[26] + 1.0*y[7]*y[33] + y[6]*y[26]; 
data[28] = u[14]; 
data[29] = u[15]; 
data[30] = u[16]; 
data[31] = u[18]; 
data[32] = u[19]; 
data[33] = u[20]; 
data[34] = u[12]; 
data[35] = u[11]; 
data[36] = x[0]; 
data[37] = 1.4142135623730951*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2)); 
data[38] = 1.4142135623730951*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2)); 
data[39] = 1.4142135623730951*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2)); 
data[40] = pow(p[18], 2)/p[12]; 
data[41] = (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]; 
data[42] = y[29]; 
data[43] = y[47]/u[22]; 
data[44] = 4.308339024928669e-5*p[42]*u[23]*log(x[9]*sqrt(x[11])/x[10]); 
data[45] = y[47]*(x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
)))); 
data[46] = x[12] + u[24] + (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   1.0
)
: (((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0) ? (
   250.0
)
: (
   (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47]
))); 
data[47] = x[13]; 
data[48] = u[26]; 

}

void de_jac_run_up_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[28]; 
data[130] = -1/p[19]; 
data[178] = 1/(p[12]*p[19]); 
data[195] = -1/p[20]; 
data[260] = -1/p[21]; 
data[325] = -1/p[21]; 
data[390] = -1/p[21]; 
data[455] = -1/p[21]; 
data[520] = -1/p[38]; 
data[524] = 2*p[32]/(p[38]*p[37]); 
data[584] = 1/(p[33]*p[39]); 
data[585] = -1/p[39]; 
data[588] = -2*p[32]/(p[33]*p[39]); 
data[650] = -1/p[40]; 
data[652] = 2*p[32]/(p[34]*p[40]); 
data[712] = 1/(p[35]*p[36]*p[41]); 
data[715] = -1/p[41]; 
data[716] = -p[32]/(p[35]*p[41]); 
data[845] = -p[47]; 
data[1950] = -p[1]; 
data[1954] = 1.0*p[0]; 
data[2015] = -p[1]; 
data[2019] = 1.0*p[0]; 
data[2080] = -p[1]; 
data[2084] = 1.0*p[0]; 
data[2145] = -p[3]; 
data[2149] = 1.0*p[2]; 
data[2206] = -1.0*p[0]; 
data[2210] = -1.0*p[1]; 
data[2271] = -1.0*p[0]; 
data[2275] = -1.0*p[1]; 
data[2336] = -1.0*p[0]; 
data[2340] = -1.0*p[1]; 
data[2401] = -1.0*p[2]; 
data[2405] = -1.0*p[3]; 
data[2470] = -p[7]; 
data[2477] = 1.0*p[6]; 
data[2535] = -p[7]; 
data[2542] = 1.0*p[6]; 
data[2600] = -p[7]; 
data[2607] = 1.0*p[6]; 
data[2665] = -p[9]; 
data[2672] = 1.0*p[8]; 
data[2684] = 0.40824829046386307*u[5]*cos(u[7]); 
data[2730] = p[11]/(pow(p[11], 2) + 1.0*pow(p[10], 2)); 
data[2737] = 1.0*p[10]/(pow(p[11], 2) + 1.0*pow(p[10], 2)); 
data[2790] = -1.0*p[6]; 
data[2797] = -1.0*p[7]; 
data[2855] = -1.0*p[6]; 
data[2862] = -1.0*p[7]; 
data[2920] = -1.0*p[6]; 
data[2927] = -1.0*p[7]; 
data[2985] = -1.0*p[8]; 
data[2992] = -1.0*p[9]; 
data[3004] = 0.40824829046386307*u[5]*sin(u[7]); 
data[3050] = -1.0*p[10]/(pow(p[11], 2) + 1.0*pow(p[10], 2)); 
data[3057] = 1.0*p[11]/(pow(p[11], 2) + 1.0*pow(p[10], 2)); 
data[3713] = p[23]/p[24]; 
data[3714] = -p[23]; 
data[3771] = p[23]; 
data[3789] = p[27]; 
data[3834] = -1/p[22]; 
data[3916] = -p[42]*p[44]; 
data[4026] = p[12]/(p[12] + 1000.0); 
data[4045] = p[46]; 
data[4094] = -p[45]; 

}

void de_jac_run_xy_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[555] = 2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   (2*p[29]*y[29]/pow(u[22], 2) + p[30]*(((y[29]/u[22]) > 0) - ((y[29]/u[22]) < 0))/u[22] + 1)/y[47]
))/(p[38]*p[37]); 
data[573] = 2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   -(p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/pow(y[47], 2)
))/(p[38]*p[37]); 
data[619] = -2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   (2*p[29]*y[29]/pow(u[22], 2) + p[30]*(((y[29]/u[22]) > 0) - ((y[29]/u[22]) < 0))/u[22] + 1)/y[47]
))/(p[33]*p[39]); 
data[637] = -2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   -(p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/pow(y[47], 2)
))/(p[33]*p[39]); 
data[683] = 2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   (2*p[29]*y[29]/pow(u[22], 2) + p[30]*(((y[29]/u[22]) > 0) - ((y[29]/u[22]) < 0))/u[22] + 1)/y[47]
))/(p[34]*p[40]); 
data[701] = 2*p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   -(p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/pow(y[47], 2)
))/(p[34]*p[40]); 
data[747] = -p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   (2*p[29]*y[29]/pow(u[22], 2) + p[30]*(((y[29]/u[22]) > 0) - ((y[29]/u[22]) < 0))/u[22] + 1)/y[47]
))/(p[35]*p[41]); 
data[765] = -p[32]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   -(p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/pow(y[47], 2)
))/(p[35]*p[41]); 
data[2484] = -0.40824829046386307*y[41]*y[46]*sin(u[6] + y[38]); 
data[2487] = 0.40824829046386307*y[46]*cos(u[6] + y[38]); 
data[2492] = 0.40824829046386307*y[41]*cos(u[6] + y[38]); 
data[2549] = -0.40824829046386307*y[42]*y[46]*sin(u[6] + y[39]); 
data[2552] = 0.40824829046386307*y[46]*cos(u[6] + y[39]); 
data[2556] = 0.40824829046386307*y[42]*cos(u[6] + y[39]); 
data[2614] = -0.40824829046386307*y[43]*y[46]*sin(u[6] + y[40]); 
data[2617] = 0.40824829046386307*y[46]*cos(u[6] + y[40]); 
data[2620] = 0.40824829046386307*y[43]*cos(u[6] + y[40]); 
data[2804] = 0.40824829046386307*y[41]*y[46]*cos(u[6] + y[38]); 
data[2807] = 0.40824829046386307*y[46]*sin(u[6] + y[38]); 
data[2812] = 0.40824829046386307*y[41]*sin(u[6] + y[38]); 
data[2869] = 0.40824829046386307*y[42]*y[46]*cos(u[6] + y[39]); 
data[2872] = 0.40824829046386307*y[46]*sin(u[6] + y[39]); 
data[2876] = 0.40824829046386307*y[42]*sin(u[6] + y[39]); 
data[2934] = 0.40824829046386307*y[43]*y[46]*cos(u[6] + y[40]); 
data[2937] = 0.40824829046386307*y[46]*sin(u[6] + y[40]); 
data[2940] = 0.40824829046386307*y[43]*sin(u[6] + y[40]); 
data[3110] = 2*p[13]*y[24] + p[14]*y[24]/sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[41]*y[46]*cos(u[6] + y[38]); 
data[3111] = 2*p[13]*y[25] + p[14]*y[25]/sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[42]*y[46]*cos(u[6] + y[39]); 
data[3112] = 2*p[13]*y[26] + p[14]*y[26]/sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[43]*y[46]*cos(u[6] + y[40]) + 0.40824829046386307*u[5]*y[46]*cos(u[7]); 
data[3113] = 2*p[13]*y[27] + p[14]*y[27]/sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7); 
data[3117] = 2*p[13]*y[31] + p[14]*y[31]/sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[41]*y[46]*sin(u[6] + y[38]); 
data[3118] = 2*p[13]*y[32] + p[14]*y[32]/sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[42]*y[46]*sin(u[6] + y[39]); 
data[3119] = 2*p[13]*y[33] + p[14]*y[33]/sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[43]*y[46]*sin(u[6] + y[40]) + 0.40824829046386307*u[5]*y[46]*sin(u[7]); 
data[3120] = 2*p[13]*y[34] + p[14]*y[34]/sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7); 
data[3124] = 0.40824829046386307*y[31]*y[41]*y[46]*cos(u[6] + y[38]) - 0.40824829046386307*y[24]*y[41]*y[46]*sin(u[6] + y[38]); 
data[3125] = 0.40824829046386307*y[32]*y[42]*y[46]*cos(u[6] + y[39]) - 0.40824829046386307*y[25]*y[42]*y[46]*sin(u[6] + y[39]); 
data[3126] = 0.40824829046386307*y[33]*y[43]*y[46]*cos(u[6] + y[40]) - 0.40824829046386307*y[26]*y[43]*y[46]*sin(u[6] + y[40]); 
data[3127] = 0.40824829046386307*y[31]*y[46]*sin(u[6] + y[38]) + 0.40824829046386307*y[24]*y[46]*cos(u[6] + y[38]); 
data[3128] = 0.40824829046386307*y[32]*y[46]*sin(u[6] + y[39]) + 0.40824829046386307*y[25]*y[46]*cos(u[6] + y[39]); 
data[3129] = 0.40824829046386307*y[33]*y[46]*sin(u[6] + y[40]) + 0.40824829046386307*y[26]*y[46]*cos(u[6] + y[40]); 
data[3132] = 0.40824829046386307*y[31]*y[41]*sin(u[6] + y[38]) + 0.40824829046386307*y[24]*y[41]*cos(u[6] + y[38]) + 0.40824829046386307*y[32]*y[42]*sin(u[6] + y[39]) + 0.40824829046386307*y[25]*y[42]*cos(u[6] + y[39]) + 0.40824829046386307*y[33]*y[43]*sin(u[6] + y[40]) + 0.40824829046386307*y[33]*u[5]*sin(u[7]) + 0.40824829046386307*y[26]*y[43]*cos(u[6] + y[40]) + 0.40824829046386307*y[26]*u[5]*cos(u[7]); 
data[3179] = 1.0/y[46]; 
data[3196] = -y[29]/pow(y[46], 2); 
data[3214] = 0.33333333333333331*y[24] - 0.28867513459481287*y[32] - 0.16666666666666657*y[25] + 0.28867513459481275*y[33] - 0.16666666666666677*y[26]; 
data[3215] = 0.33333333333333331*y[31] - 0.16666666666666657*y[32] + 0.28867513459481287*y[25] - 0.16666666666666677*y[33] - 0.28867513459481275*y[26]; 
data[3216] = 0.28867513459481287*y[31] - 0.16666666666666657*y[24] + 0.33333333333333326*y[25] - 0.28867513459481281*y[33] - 0.16666666666666652*y[26]; 
data[3217] = -0.16666666666666657*y[31] - 0.28867513459481287*y[24] + 0.33333333333333326*y[32] - 0.16666666666666652*y[33] + 0.28867513459481281*y[26]; 
data[3218] = -0.28867513459481275*y[31] - 0.16666666666666677*y[24] + 0.28867513459481281*y[32] - 0.16666666666666652*y[25] + 0.33333333333333326*y[26]; 
data[3219] = -0.16666666666666677*y[31] + 0.28867513459481275*y[24] - 0.16666666666666652*y[32] - 0.28867513459481281*y[25] + 0.33333333333333326*y[33]; 
data[3238] = 0.33333333333333331*y[0] - 0.28867513459481287*y[3] - 0.16666666666666657*y[2] + 0.28867513459481275*y[5] - 0.16666666666666677*y[4]; 
data[3239] = 0.28867513459481287*y[1] - 0.16666666666666657*y[0] + 0.33333333333333326*y[2] - 0.28867513459481281*y[5] - 0.16666666666666652*y[4]; 
data[3240] = -0.28867513459481275*y[1] - 0.16666666666666677*y[0] + 0.28867513459481281*y[3] - 0.16666666666666652*y[2] + 0.33333333333333326*y[4]; 
data[3245] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[3246] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[3247] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[3278] = -0.33333333333333331*y[31] + 0.16666666666666657*y[32] - 0.28867513459481287*y[25] + 0.16666666666666677*y[33] + 0.28867513459481275*y[26]; 
data[3279] = 0.33333333333333331*y[24] - 0.28867513459481287*y[32] - 0.16666666666666657*y[25] + 0.28867513459481275*y[33] - 0.16666666666666677*y[26]; 
data[3280] = 0.16666666666666657*y[31] + 0.28867513459481287*y[24] - 0.33333333333333326*y[32] + 0.16666666666666652*y[33] - 0.28867513459481281*y[26]; 
data[3281] = 0.28867513459481287*y[31] - 0.16666666666666657*y[24] + 0.33333333333333326*y[25] - 0.28867513459481281*y[33] - 0.16666666666666652*y[26]; 
data[3282] = 0.16666666666666677*y[31] - 0.28867513459481275*y[24] + 0.16666666666666652*y[32] + 0.28867513459481281*y[25] - 0.33333333333333326*y[33]; 
data[3283] = -0.28867513459481275*y[31] - 0.16666666666666677*y[24] + 0.28867513459481281*y[32] - 0.16666666666666652*y[25] + 0.33333333333333326*y[26]; 
data[3302] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[3303] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[3304] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[3309] = -0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4]; 
data[3310] = -0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4]; 
data[3311] = 0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4]; 
data[3328] = -(x[4] + u[14])*(p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) - 1.0*(x[4] + u[14])*sin(u[8] + x[0]))*sin(u[8] + x[0])/(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2)) + 1.0*(x[4] + u[14])*(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]))*cos(u[8] + x[0])/(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2)); 
data[3332] = (p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) - 1.0*(x[4] + u[14])*sin(u[8] + x[0]))*cos(u[8] + x[0])/(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2)) + 1.0*(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]))*sin(u[8] + x[0])/(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2)); 
data[3366] = -p[17]*p[12]*(p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) - 1.0*(x[4] + u[14])*sin(u[8] + x[0]))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2))) - 1.0*p[12]*p[16]*(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2))); 
data[3373] = -1.0*p[17]*p[12]*(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2))) + 1.0*p[12]*p[16]*(p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) - 1.0*(x[4] + u[14])*sin(u[8] + x[0]))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[31] + 1.0*p[16]*y[24])/pow(p[18], 2) + 1.0*(x[4] + u[14])*sin(u[8] + x[0]), 2) + pow(-p[12]*(p[17]*y[24] - 1.0*p[16]*y[31])/pow(p[18], 2) + (x[4] + u[14])*cos(u[8] + x[0]), 2))); 
data[3392] = -(x[5] + u[15])*(p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) - 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953))*sin(u[9] + x[0] - 2.0943951023931953)/(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2)) + 1.0*(x[5] + u[15])*(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953))*cos(u[9] + x[0] - 2.0943951023931953)/(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2)); 
data[3397] = (p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) - 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953))*cos(u[9] + x[0] - 2.0943951023931953)/(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2)) + 1.0*(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953))*sin(u[9] + x[0] - 2.0943951023931953)/(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2)); 
data[3431] = -p[17]*p[12]*(p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) - 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2))) - 1.0*p[12]*p[16]*(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3438] = -1.0*p[17]*p[12]*(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2))) + 1.0*p[12]*p[16]*(p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) - 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[32] + 1.0*p[16]*y[25])/pow(p[18], 2) + 1.0*(x[5] + u[15])*sin(u[9] + x[0] - 2.0943951023931953), 2) + pow(-p[12]*(p[17]*y[25] - 1.0*p[16]*y[32])/pow(p[18], 2) + (x[5] + u[15])*cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3456] = -(x[6] + u[16])*(p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) - 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905))*sin(u[10] + x[0] - 4.1887902047863905)/(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2)) + 1.0*(x[6] + u[16])*(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905))*cos(u[10] + x[0] - 4.1887902047863905)/(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2)); 
data[3462] = (p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) - 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905))*cos(u[10] + x[0] - 4.1887902047863905)/(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2)) + 1.0*(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905))*sin(u[10] + x[0] - 4.1887902047863905)/(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2)); 
data[3496] = -p[17]*p[12]*(p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) - 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2))) - 1.0*p[12]*p[16]*(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3503] = -1.0*p[17]*p[12]*(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2))) + 1.0*p[12]*p[16]*(p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) - 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905))/(pow(p[18], 2)*(pow(-p[12]*(1.0*p[17]*y[33] + 1.0*p[16]*y[26])/pow(p[18], 2) + 1.0*(x[6] + u[16])*sin(u[10] + x[0] - 4.1887902047863905), 2) + pow(-p[12]*(p[17]*y[26] - 1.0*p[16]*y[33])/pow(p[18], 2) + (x[6] + u[16])*cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3520] = 3.4641016151377544*(-1.0/2.0*x[4]*p[17]*p[12]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) + (1.0/2.0)*x[4]*p[17]*p[12]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*x[4]*p[12]*p[16]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*x[4]*p[12]*p[16]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) + (1.0/2.0)*p[17]*p[12]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))); 
data[3524] = 3.4641016151377544*(0.5*x[4]*pow(sin(u[8] + x[0]), 2) + 0.5*x[4]*pow(cos(u[8] + x[0]), 2) - 1.0/2.0*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + (1.0/2.0)*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + (1.0/2.0)*u[14]*pow(sin(u[8] + x[0]), 2) + (1.0/2.0)*u[14]*pow(cos(u[8] + x[0]), 2))/(y[46]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))); 
data[3558] = 3.4641016151377544*(-1.0/2.0*x[4]*p[17]*p[12]*cos(u[8] + x[0])/pow(p[18], 2) - 1.0/2.0*x[4]*p[12]*p[16]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[24]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[14]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[24]/pow(p[18], 4) - 1.0/2.0*p[12]*p[16]*u[14]*sin(u[8] + x[0])/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))); 
data[3565] = 3.4641016151377544*(-1.0/2.0*x[4]*p[17]*p[12]*sin(u[8] + x[0])/pow(p[18], 2) + (1.0/2.0)*x[4]*p[12]*p[16]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[31]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[14]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[31]/pow(p[18], 4) + (1.0/2.0)*p[12]*p[16]*u[14]*cos(u[8] + x[0])/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))); 
data[3580] = -3.4641016151377544*sqrt(0.5*pow(x[4], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[8] + x[0]), 2) - x[4]*p[17]*p[12]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - x[4]*p[17]*p[12]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + x[4]*p[12]*p[16]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - x[4]*p[12]*p[16]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + x[4]*u[14]*pow(sin(u[8] + x[0]), 2) + x[4]*u[14]*pow(cos(u[8] + x[0]), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[24], 2)/pow(p[18], 4) - p[17]*p[12]*u[14]*y[31]*sin(u[8] + x[0])/pow(p[18], 2) - p[17]*p[12]*u[14]*y[24]*cos(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[31], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[24], 2)/pow(p[18], 4) + p[12]*p[16]*u[14]*y[31]*cos(u[8] + x[0])/pow(p[18], 2) - p[12]*p[16]*u[14]*y[24]*sin(u[8] + x[0])/pow(p[18], 2) + 0.5*pow(u[14], 2)*pow(sin(u[8] + x[0]), 2) + 0.5*pow(u[14], 2)*pow(cos(u[8] + x[0]), 2))/pow(y[46], 2); 
data[3584] = 3.4641016151377544*(-1.0/2.0*x[5]*p[17]*p[12]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + (1.0/2.0)*x[5]*p[17]*p[12]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*x[5]*p[12]*p[16]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*x[5]*p[12]*p[16]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + (1.0/2.0)*p[17]*p[12]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3589] = 3.4641016151377544*(0.5*x[5]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*x[5]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - 1.0/2.0*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + (1.0/2.0)*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + (1.0/2.0)*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + (1.0/2.0)*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))/(y[46]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3623] = 3.4641016151377544*(-1.0/2.0*x[5]*p[17]*p[12]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - 1.0/2.0*x[5]*p[12]*p[16]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[25]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[15]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[25]/pow(p[18], 4) - 1.0/2.0*p[12]*p[16]*u[15]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3630] = 3.4641016151377544*(-1.0/2.0*x[5]*p[17]*p[12]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + (1.0/2.0)*x[5]*p[12]*p[16]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[32]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[15]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[32]/pow(p[18], 4) + (1.0/2.0)*p[12]*p[16]*u[15]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))); 
data[3644] = -3.4641016151377544*sqrt(0.5*pow(x[5], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) - x[5]*p[17]*p[12]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[17]*p[12]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*p[12]*p[16]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - x[5]*p[12]*p[16]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + x[5]*u[15]*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + x[5]*u[15]*pow(cos(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[25], 2)/pow(p[18], 4) - p[17]*p[12]*u[15]*y[32]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[17]*p[12]*u[15]*y[25]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[32], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[25], 2)/pow(p[18], 4) + p[12]*p[16]*u[15]*y[32]*cos(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) - p[12]*p[16]*u[15]*y[25]*sin(u[9] + x[0] - 2.0943951023931953)/pow(p[18], 2) + 0.5*pow(u[15], 2)*pow(sin(u[9] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[15], 2)*pow(cos(u[9] + x[0] - 2.0943951023931953), 2))/pow(y[46], 2); 
data[3648] = 3.4641016151377544*(-1.0/2.0*x[6]*p[17]*p[12]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + (1.0/2.0)*x[6]*p[17]*p[12]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*x[6]*p[12]*p[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*x[6]*p[12]*p[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + (1.0/2.0)*p[17]*p[12]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3654] = 3.4641016151377544*(0.5*x[6]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*x[6]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - 1.0/2.0*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + (1.0/2.0)*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + (1.0/2.0)*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + (1.0/2.0)*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))/(y[46]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3688] = 3.4641016151377544*(-1.0/2.0*x[6]*p[17]*p[12]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - 1.0/2.0*x[6]*p[12]*p[16]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[26]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[16]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[26]/pow(p[18], 4) - 1.0/2.0*p[12]*p[16]*u[16]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3695] = 3.4641016151377544*(-1.0/2.0*x[6]*p[17]*p[12]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + (1.0/2.0)*x[6]*p[12]*p[16]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*y[33]/pow(p[18], 4) - 1.0/2.0*p[17]*p[12]*u[16]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*y[33]/pow(p[18], 4) + (1.0/2.0)*p[12]*p[16]*u[16]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2))/(y[46]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))); 
data[3708] = -3.4641016151377544*sqrt(0.5*pow(x[6], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) - x[6]*p[17]*p[12]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[17]*p[12]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*p[12]*p[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - x[6]*p[12]*p[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + x[6]*u[16]*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + x[6]*u[16]*pow(cos(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[17], 2)*pow(p[12], 2)*pow(y[26], 2)/pow(p[18], 4) - p[17]*p[12]*u[16]*y[33]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[17]*p[12]*u[16]*y[26]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[33], 2)/pow(p[18], 4) + 0.5*pow(p[12], 2)*pow(p[16], 2)*pow(y[26], 2)/pow(p[18], 4) + p[12]*p[16]*u[16]*y[33]*cos(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) - p[12]*p[16]*u[16]*y[26]*sin(u[10] + x[0] - 4.1887902047863905)/pow(p[18], 2) + 0.5*pow(u[16], 2)*pow(sin(u[10] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[16], 2)*pow(cos(u[10] + x[0] - 4.1887902047863905), 2))/pow(y[46], 2); 
data[3913] = 4.308339024928669e-5*p[42]*u[23]/x[9]; 
data[3914] = -4.308339024928669e-5*p[42]*u[23]/x[10]; 
data[3915] = 2.1541695124643345e-5*p[42]*u[23]/x[11]; 
data[3947] = -p[42]*p[44]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   (2*p[29]*y[29]/pow(u[22], 2) + p[30]*(((y[29]/u[22]) > 0) - ((y[29]/u[22]) < 0))/u[22] + 1)/y[47]
)); 
data[3965] = -p[42]*p[44]*(((p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] > 250.0 || (p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/y[47] < 1.0) ? (
   0
)
: (
   -(p[29]*pow(y[29], 2)/pow(u[22], 2) + p[30]*fabs(y[29]/u[22]) + p[31] + y[29])/pow(y[47], 2)
)) - 1; 

}

void de_jac_run_num_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[58] = 314.15926535897933; 
data[62] = -314.15926535897933; 
data[66] = -1; 
data[123] = 1; 
data[780] = -9.9999999999999995e-7; 
data[894] = -1; 
data[910] = 184.75059629806532; 
data[911] = 94.615102344455565; 
data[912] = -69.368640085185334; 
data[913] = -17.974903844364775; 
data[914] = -43.675799641021136; 
data[915] = -40.74868784166955; 
data[916] = -69.368640085185348; 
data[917] = -17.974903844364817; 
data[918] = -184.75059629806532; 
data[919] = -94.615102344455565; 
data[920] = 69.368640085185334; 
data[921] = 17.974903844364775; 
data[922] = 43.675799641021136; 
data[923] = 40.74868784166955; 
data[924] = 69.368640085185348; 
data[925] = 17.974903844364817; 
data[934] = -1; 
data[974] = -94.615102344455565; 
data[975] = 184.75059629806532; 
data[976] = 17.974903844364775; 
data[977] = -69.368640085185334; 
data[978] = 40.74868784166955; 
data[979] = -43.675799641021136; 
data[980] = 17.974903844364817; 
data[981] = -69.368640085185348; 
data[982] = 94.615102344455565; 
data[983] = -184.75059629806532; 
data[984] = -17.974903844364775; 
data[985] = 69.368640085185334; 
data[986] = -40.74868784166955; 
data[987] = 43.675799641021136; 
data[988] = -17.974903844364817; 
data[989] = 69.368640085185348; 
data[1005] = -1; 
data[1038] = -69.368640085185319; 
data[1039] = -17.974903844364796; 
data[1040] = 184.75059629806523; 
data[1041] = 94.615102344455607; 
data[1042] = -69.368640085185291; 
data[1043] = -17.974903844364814; 
data[1044] = -43.675799641021079; 
data[1045] = -40.748687841669586; 
data[1046] = 69.368640085185319; 
data[1047] = 17.974903844364796; 
data[1048] = -184.75059629806523; 
data[1049] = -94.615102344455607; 
data[1050] = 69.368640085185291; 
data[1051] = 17.974903844364814; 
data[1052] = 43.675799641021079; 
data[1053] = 40.748687841669586; 
data[1063] = -1; 
data[1102] = 17.974903844364796; 
data[1103] = -69.368640085185319; 
data[1104] = -94.615102344455607; 
data[1105] = 184.75059629806523; 
data[1106] = 17.974903844364814; 
data[1107] = -69.368640085185291; 
data[1108] = 40.748687841669586; 
data[1109] = -43.675799641021079; 
data[1110] = -17.974903844364796; 
data[1111] = 69.368640085185319; 
data[1112] = 94.615102344455607; 
data[1113] = -184.75059629806523; 
data[1114] = -17.974903844364814; 
data[1115] = 69.368640085185291; 
data[1116] = -40.748687841669586; 
data[1117] = 43.675799641021079; 
data[1134] = -1; 
data[1166] = -43.675799641021122; 
data[1167] = -40.748687841669557; 
data[1168] = -69.368640085185277; 
data[1169] = -17.97490384436481; 
data[1170] = 184.75059629806526; 
data[1171] = 94.615102344455636; 
data[1172] = -69.368640085185277; 
data[1173] = -17.974903844364817; 
data[1174] = 43.675799641021122; 
data[1175] = 40.748687841669557; 
data[1176] = 69.368640085185277; 
data[1177] = 17.97490384436481; 
data[1178] = -184.75059629806526; 
data[1179] = -94.615102344455636; 
data[1180] = 69.368640085185277; 
data[1181] = 17.974903844364817; 
data[1192] = -1; 
data[1230] = 40.748687841669557; 
data[1231] = -43.675799641021122; 
data[1232] = 17.97490384436481; 
data[1233] = -69.368640085185277; 
data[1234] = -94.615102344455636; 
data[1235] = 184.75059629806526; 
data[1236] = 17.974903844364817; 
data[1237] = -69.368640085185277; 
data[1238] = -40.748687841669557; 
data[1239] = 43.675799641021122; 
data[1240] = -17.97490384436481; 
data[1241] = 69.368640085185277; 
data[1242] = 94.615102344455636; 
data[1243] = -184.75059629806526; 
data[1244] = -17.974903844364817; 
data[1245] = 69.368640085185277; 
data[1263] = -1; 
data[1294] = -69.368640085185334; 
data[1295] = -17.9749038443648; 
data[1296] = -43.675799641021086; 
data[1297] = -40.748687841669586; 
data[1298] = -69.368640085185291; 
data[1299] = -17.974903844364821; 
data[1300] = 184.75059629806523; 
data[1301] = 94.61510234445565; 
data[1302] = 69.368640085185334; 
data[1303] = 17.9749038443648; 
data[1304] = 43.675799641021086; 
data[1305] = 40.748687841669586; 
data[1306] = 69.368640085185291; 
data[1307] = 17.974903844364821; 
data[1308] = -184.75059629806523; 
data[1309] = -94.61510234445565; 
data[1321] = -1; 
data[1358] = 17.9749038443648; 
data[1359] = -69.368640085185334; 
data[1360] = 40.748687841669586; 
data[1361] = -43.675799641021086; 
data[1362] = 17.974903844364821; 
data[1363] = -69.368640085185291; 
data[1364] = -94.61510234445565; 
data[1365] = 184.75059629806523; 
data[1366] = -17.9749038443648; 
data[1367] = 69.368640085185334; 
data[1368] = -40.748687841669586; 
data[1369] = 43.675799641021086; 
data[1370] = -17.974903844364821; 
data[1371] = 69.368640085185291; 
data[1372] = 94.61510234445565; 
data[1373] = -184.75059629806523; 
data[1392] = -1; 
data[1422] = -184.75059629806532; 
data[1423] = -94.615102344455565; 
data[1424] = 69.368640085185334; 
data[1425] = 17.974903844364775; 
data[1426] = 43.675799641021136; 
data[1427] = 40.74868784166955; 
data[1428] = 69.368640085185348; 
data[1429] = 17.974903844364817; 
data[1430] = 184.75059629806532; 
data[1431] = 94.615102344455565; 
data[1432] = -69.368640085185334; 
data[1433] = -17.974903844364775; 
data[1434] = -43.675799641021136; 
data[1435] = -40.74868784166955; 
data[1436] = -69.368640085185348; 
data[1437] = -17.974903844364817; 
data[1438] = -1; 
data[1486] = 94.615102344455565; 
data[1487] = -184.75059629806532; 
data[1488] = -17.974903844364775; 
data[1489] = 69.368640085185334; 
data[1490] = -40.74868784166955; 
data[1491] = 43.675799641021136; 
data[1492] = -17.974903844364817; 
data[1493] = 69.368640085185348; 
data[1494] = -94.615102344455565; 
data[1495] = 184.75059629806532; 
data[1496] = 17.974903844364775; 
data[1497] = -69.368640085185334; 
data[1498] = 40.74868784166955; 
data[1499] = -43.675799641021136; 
data[1500] = 17.974903844364817; 
data[1501] = -69.368640085185348; 
data[1506] = -1; 
data[1550] = 69.368640085185319; 
data[1551] = 17.974903844364796; 
data[1552] = -184.75059629806523; 
data[1553] = -94.615102344455607; 
data[1554] = 69.368640085185291; 
data[1555] = 17.974903844364814; 
data[1556] = 43.675799641021079; 
data[1557] = 40.748687841669586; 
data[1558] = -69.368640085185319; 
data[1559] = -17.974903844364796; 
data[1560] = 184.75059629806523; 
data[1561] = 94.615102344455607; 
data[1562] = -69.368640085185291; 
data[1563] = -17.974903844364814; 
data[1564] = -43.675799641021079; 
data[1565] = -40.748687841669586; 
data[1567] = -1; 
data[1614] = -17.974903844364796; 
data[1615] = 69.368640085185319; 
data[1616] = 94.615102344455607; 
data[1617] = -184.75059629806523; 
data[1618] = -17.974903844364814; 
data[1619] = 69.368640085185291; 
data[1620] = -40.748687841669586; 
data[1621] = 43.675799641021079; 
data[1622] = 17.974903844364796; 
data[1623] = -69.368640085185319; 
data[1624] = -94.615102344455607; 
data[1625] = 184.75059629806523; 
data[1626] = 17.974903844364814; 
data[1627] = -69.368640085185291; 
data[1628] = 40.748687841669586; 
data[1629] = -43.675799641021079; 
data[1635] = -1; 
data[1678] = 43.675799641021122; 
data[1679] = 40.748687841669557; 
data[1680] = 69.368640085185277; 
data[1681] = 17.97490384436481; 
data[1682] = -184.75059629806526; 
data[1683] = -94.615102344455636; 
data[1684] = 69.368640085185277; 
data[1685] = 17.974903844364817; 
data[1686] = -43.675799641021122; 
data[1687] = -40.748687841669557; 
data[1688] = -69.368640085185277; 
data[1689] = -17.97490384436481; 
data[1690] = 184.75059629806526; 
data[1691] = 94.615102344455636; 
data[1692] = -69.368640085185277; 
data[1693] = -17.974903844364817; 
data[1696] = -1; 
data[1742] = -40.748687841669557; 
data[1743] = 43.675799641021122; 
data[1744] = -17.97490384436481; 
data[1745] = 69.368640085185277; 
data[1746] = 94.615102344455636; 
data[1747] = -184.75059629806526; 
data[1748] = -17.974903844364817; 
data[1749] = 69.368640085185277; 
data[1750] = 40.748687841669557; 
data[1751] = -43.675799641021122; 
data[1752] = 17.97490384436481; 
data[1753] = -69.368640085185277; 
data[1754] = -94.615102344455636; 
data[1755] = 184.75059629806526; 
data[1756] = 17.974903844364817; 
data[1757] = -69.368640085185277; 
data[1764] = -1; 
data[1806] = 69.368640085185334; 
data[1807] = 17.9749038443648; 
data[1808] = 43.675799641021086; 
data[1809] = 40.748687841669586; 
data[1810] = 69.368640085185291; 
data[1811] = 17.974903844364821; 
data[1812] = -184.75059629806523; 
data[1813] = -94.61510234445565; 
data[1814] = -69.368640085185334; 
data[1815] = -17.9749038443648; 
data[1816] = -43.675799641021086; 
data[1817] = -40.748687841669586; 
data[1818] = -69.368640085185291; 
data[1819] = -17.974903844364821; 
data[1820] = 184.75059629806523; 
data[1821] = 94.61510234445565; 
data[1825] = -1; 
data[1870] = -17.9749038443648; 
data[1871] = 69.368640085185334; 
data[1872] = -40.748687841669586; 
data[1873] = 43.675799641021086; 
data[1874] = -17.974903844364821; 
data[1875] = 69.368640085185291; 
data[1876] = 94.61510234445565; 
data[1877] = -184.75059629806523; 
data[1878] = 17.9749038443648; 
data[1879] = -69.368640085185334; 
data[1880] = 40.748687841669586; 
data[1881] = -43.675799641021086; 
data[1882] = 17.974903844364821; 
data[1883] = -69.368640085185291; 
data[1884] = -94.61510234445565; 
data[1885] = 184.75059629806523; 
data[1893] = -1; 
data[1942] = -1; 
data[2008] = -1; 
data[2074] = -1; 
data[2140] = -1; 
data[2199] = -1.0; 
data[2265] = -1.0; 
data[2331] = -1.0; 
data[2397] = -1.0; 
data[2446] = -1; 
data[2474] = 1; 
data[2512] = -1; 
data[2538] = 1; 
data[2578] = -1; 
data[2602] = 1; 
data[2644] = -1; 
data[2666] = 1; 
data[2726] = 1; 
data[2727] = 1; 
data[2728] = 1; 
data[2729] = 1; 
data[2767] = -1.0; 
data[2801] = 1.0; 
data[2833] = -1.0; 
data[2865] = 1.0; 
data[2899] = -1.0; 
data[2929] = 1.0; 
data[2965] = -1.0; 
data[2993] = 1.0; 
data[3053] = 1.0; 
data[3054] = 1.0; 
data[3055] = 1.0; 
data[3056] = 1.0; 
data[3115] = -1; 
data[3180] = -1; 
data[3250] = -1; 
data[3315] = -1; 
data[3380] = -1; 
data[3445] = -1; 
data[3510] = -1; 
data[3575] = -1; 
data[3640] = -1; 
data[3705] = -1; 
data[3770] = -1; 
data[3779] = 1; 
data[3835] = -1; 
data[3900] = -1; 
data[4030] = -1; 
data[4095] = -1; 

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_jac_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_num_eval _cffi_d_de_jac_run_num_eval
#endif

static void _cffi_d_de_jac_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_up_eval _cffi_d_de_jac_run_up_eval
#endif

static void _cffi_d_de_jac_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_xy_eval _cffi_d_de_jac_run_xy_eval
#endif

static void _cffi_d_f_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_run_eval _cffi_d_f_run_eval
#endif

static void _cffi_d_g_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_run_eval _cffi_d_g_run_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_jac_run_num_eval", (void *)_cffi_f_de_jac_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_num_eval },
  { "de_jac_run_up_eval", (void *)_cffi_f_de_jac_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_up_eval },
  { "de_jac_run_xy_eval", (void *)_cffi_f_de_jac_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_xy_eval },
  { "f_run_eval", (void *)_cffi_f_f_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_run_eval },
  { "g_run_eval", (void *)_cffi_f_g_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_run_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  6,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_run_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_run_cffi(void) { return NULL; }
#  else
     inittemp_run_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_run_cffi(void)
{
  return _cffi_init("temp_run_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_run_cffi(void)
{
  _cffi_init("temp_run_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
