#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_ini_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[23]*x[0] + 314.15926535897933*y[44] - 314.15926535897933*y[46]; 
data[1] = -x[2] + y[45]; 
data[2] = (-x[2] + y[36]/p[7])/p[14]; 
data[3] = (u[20] - x[3])/p[15]; 
data[4] = (-x[4] + u[26])/p[16]; 
data[5] = (-x[5] + u[27])/p[16]; 
data[6] = (-x[6] + u[28])/p[16]; 
data[7] = (-x[7] + u[29])/p[16]; 
data[8] = -p[26]*x[8] - y[46] + u[30] + 1; 

}

void g_ini_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14] - y[24]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14] - y[31]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14] - y[25]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14] - y[32]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14] - y[26]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14] - y[33]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14] - y[27]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14] - y[34]; 
data[8] = -94.615102344455565*y[1] - 184.75059629806532*y[0] + 17.974903844364775*y[3] + 69.368640085185334*y[2] + 40.74868784166955*y[5] + 43.675799641021136*y[4] + 17.974903844364817*y[7] + 69.368640085185348*y[6] + 94.615102344455565*y[9] + 184.75059629806532*y[8] - 17.974903844364775*y[11] - 69.368640085185334*y[10] - 40.74868784166955*y[13] - 43.675799641021136*y[12] - 17.974903844364817*y[15] - 69.368640085185348*y[14] - y[16]; 
data[9] = -184.75059629806532*y[1] + 94.615102344455565*y[0] + 69.368640085185334*y[3] - 17.974903844364775*y[2] + 43.675799641021136*y[5] - 40.74868784166955*y[4] + 69.368640085185348*y[7] - 17.974903844364817*y[6] + 184.75059629806532*y[9] - 94.615102344455565*y[8] - 69.368640085185334*y[11] + 17.974903844364775*y[10] - 43.675799641021136*y[13] + 40.74868784166955*y[12] - 69.368640085185348*y[15] + 17.974903844364817*y[14] - y[17]; 
data[10] = 17.974903844364796*y[1] + 69.368640085185319*y[0] - 94.615102344455607*y[3] - 184.75059629806523*y[2] + 17.974903844364814*y[5] + 69.368640085185291*y[4] + 40.748687841669586*y[7] + 43.675799641021079*y[6] - 17.974903844364796*y[9] - 69.368640085185319*y[8] + 94.615102344455607*y[11] + 184.75059629806523*y[10] - 17.974903844364814*y[13] - 69.368640085185291*y[12] - 40.748687841669586*y[15] - 43.675799641021079*y[14] - y[18]; 
data[11] = 69.368640085185319*y[1] - 17.974903844364796*y[0] - 184.75059629806523*y[3] + 94.615102344455607*y[2] + 69.368640085185291*y[5] - 17.974903844364814*y[4] + 43.675799641021079*y[7] - 40.748687841669586*y[6] - 69.368640085185319*y[9] + 17.974903844364796*y[8] + 184.75059629806523*y[11] - 94.615102344455607*y[10] - 69.368640085185291*y[13] + 17.974903844364814*y[12] - 43.675799641021079*y[15] + 40.748687841669586*y[14] - y[19]; 
data[12] = 40.748687841669557*y[1] + 43.675799641021122*y[0] + 17.97490384436481*y[3] + 69.368640085185277*y[2] - 94.615102344455636*y[5] - 184.75059629806526*y[4] + 17.974903844364817*y[7] + 69.368640085185277*y[6] - 40.748687841669557*y[9] - 43.675799641021122*y[8] - 17.97490384436481*y[11] - 69.368640085185277*y[10] + 94.615102344455636*y[13] + 184.75059629806526*y[12] - 17.974903844364817*y[15] - 69.368640085185277*y[14] - y[20]; 
data[13] = 43.675799641021122*y[1] - 40.748687841669557*y[0] + 69.368640085185277*y[3] - 17.97490384436481*y[2] - 184.75059629806526*y[5] + 94.615102344455636*y[4] + 69.368640085185277*y[7] - 17.974903844364817*y[6] - 43.675799641021122*y[9] + 40.748687841669557*y[8] - 69.368640085185277*y[11] + 17.97490384436481*y[10] + 184.75059629806526*y[13] - 94.615102344455636*y[12] - 69.368640085185277*y[15] + 17.974903844364817*y[14] - y[21]; 
data[14] = 17.9749038443648*y[1] + 69.368640085185334*y[0] + 40.748687841669586*y[3] + 43.675799641021086*y[2] + 17.974903844364821*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.75059629806523*y[6] - 17.9749038443648*y[9] - 69.368640085185334*y[8] - 40.748687841669586*y[11] - 43.675799641021086*y[10] - 17.974903844364821*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.75059629806523*y[14] - y[22]; 
data[15] = 69.368640085185334*y[1] - 17.9749038443648*y[0] + 43.675799641021086*y[3] - 40.748687841669586*y[2] + 69.368640085185291*y[5] - 17.974903844364821*y[4] - 184.75059629806523*y[7] + 94.61510234445565*y[6] - 69.368640085185334*y[9] + 17.9749038443648*y[8] - 43.675799641021086*y[11] + 40.748687841669586*y[10] - 69.368640085185291*y[13] + 17.974903844364821*y[12] + 184.75059629806523*y[15] - 94.61510234445565*y[14] - y[23]; 
data[16] = p[0]*(pow(y[9], 2)*u[2] - 2*y[9]*y[15]*u[2] + pow(y[8], 2)*u[2] - 2*y[8]*y[14]*u[2] + pow(y[15], 2)*u[2] + pow(y[14], 2)*u[2] + u[0] + (y[9]*y[17] + y[8]*y[16] - y[15]*y[17] - y[14]*y[16])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[17] = p[0]*(pow(y[11], 2)*u[6] - 2*y[11]*y[15]*u[6] + pow(y[10], 2)*u[6] - 2*y[10]*y[14]*u[6] + pow(y[15], 2)*u[6] + pow(y[14], 2)*u[6] + u[4] + (y[11]*y[19] + y[10]*y[18] - y[15]*y[19] - y[14]*y[18])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[18] = p[0]*(pow(y[13], 2)*u[10] - 2*y[13]*y[15]*u[10] + pow(y[12], 2)*u[10] - 2*y[12]*y[14]*u[10] + pow(y[15], 2)*u[10] + pow(y[14], 2)*u[10] + u[8] + (y[13]*y[21] + y[12]*y[20] - y[15]*y[21] - y[14]*y[20])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[19] = p[0]*(-1.0*pow(y[9], 2)*u[3] + 2.0*y[9]*y[15]*u[3] - 1.0*pow(y[8], 2)*u[3] + 2.0*y[8]*y[14]*u[3] - 1.0*pow(y[15], 2)*u[3] - 1.0*pow(y[14], 2)*u[3] + u[1] + (y[9]*y[16] - y[8]*y[17] - y[15]*y[16] + y[14]*y[17])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[20] = p[0]*(-1.0*pow(y[11], 2)*u[7] + 2.0*y[11]*y[15]*u[7] - 1.0*pow(y[10], 2)*u[7] + 2.0*y[10]*y[14]*u[7] - 1.0*pow(y[15], 2)*u[7] - 1.0*pow(y[14], 2)*u[7] + u[5] + (y[11]*y[18] - y[10]*y[19] - y[15]*y[18] + y[14]*y[19])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[21] = p[0]*(-1.0*pow(y[13], 2)*u[11] + 2.0*y[13]*y[15]*u[11] - 1.0*pow(y[12], 2)*u[11] + 2.0*y[12]*y[14]*u[11] - 1.0*pow(y[15], 2)*u[11] - 1.0*pow(y[14], 2)*u[11] + u[9] + (y[13]*y[20] - y[12]*y[21] - y[15]*y[20] + y[14]*y[21])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[22] = y[16] + y[18] + y[20] + y[22]; 
data[23] = y[17] + y[19] + y[21] + y[23]; 
data[24] = -p[2]*y[24] - y[0] + 1.0*p[1]*y[31] + 0.40824829046386307*y[41]*u[12]*cos(u[14] + y[38]) + y[28]; 
data[25] = -p[2]*y[25] - y[2] + 1.0*p[1]*y[32] + 0.40824829046386307*y[42]*u[12]*cos(u[14] + y[39]) + y[28]; 
data[26] = -p[2]*y[26] - y[4] + 1.0*p[1]*y[33] + 0.40824829046386307*y[43]*u[12]*cos(u[14] + y[40]) + y[28]; 
data[27] = -p[4]*y[27] - y[6] + 1.0*p[3]*y[34] + 0.40824829046386307*u[13]*u[12]*cos(u[15]) + y[28]; 
data[28] = p[6]*y[28]/(pow(p[6], 2) + 1.0*pow(p[5], 2)) + 1.0*p[5]*y[35]/(pow(p[6], 2) + 1.0*pow(p[5], 2)) + y[24] + y[25] + y[26] + y[27]; 
data[29] = -1.0*p[2]*y[31] - 1.0*y[1] - 1.0*p[1]*y[24] + 0.40824829046386307*y[41]*u[12]*sin(u[14] + y[38]) + 1.0*y[35]; 
data[30] = -1.0*p[2]*y[32] - 1.0*y[3] - 1.0*p[1]*y[25] + 0.40824829046386307*y[42]*u[12]*sin(u[14] + y[39]) + 1.0*y[35]; 
data[31] = -1.0*p[2]*y[33] - 1.0*y[5] - 1.0*p[1]*y[26] + 0.40824829046386307*y[43]*u[12]*sin(u[14] + y[40]) + 1.0*y[35]; 
data[32] = -1.0*p[4]*y[34] - 1.0*y[7] - 1.0*p[3]*y[27] + 0.40824829046386307*u[13]*u[12]*sin(u[15]) + 1.0*y[35]; 
data[33] = 1.0*p[6]*y[35]/(pow(p[6], 2) + 1.0*pow(p[5], 2)) - 1.0*p[5]*y[28]/(pow(p[6], 2) + 1.0*pow(p[5], 2)) + 1.0*y[31] + 1.0*y[32] + 1.0*y[33] + 1.0*y[34]; 
data[34] = p[8]*(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + p[8]*(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + p[8]*(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + p[8]*(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7) + p[9]*sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + p[9]*sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + p[9]*sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + p[9]*sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7) + 4*p[10] + 0.40824829046386307*y[31]*y[41]*u[12]*sin(u[14] + y[38]) + 0.40824829046386307*y[24]*y[41]*u[12]*cos(u[14] + y[38]) + 0.40824829046386307*y[32]*y[42]*u[12]*sin(u[14] + y[39]) + 0.40824829046386307*y[25]*y[42]*u[12]*cos(u[14] + y[39]) + 0.40824829046386307*y[33]*y[43]*u[12]*sin(u[14] + y[40]) + 0.40824829046386307*y[33]*u[13]*u[12]*sin(u[15]) + 0.40824829046386307*y[26]*y[43]*u[12]*cos(u[14] + y[40]) + 0.40824829046386307*y[26]*u[13]*u[12]*cos(u[15]) - y[29]; 
data[35] = -y[30] + y[29]/u[12]; 
data[36] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) - y[1]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[0]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] + 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] - 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] - 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] + 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[31]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[24]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[24]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) - y[36]; 
data[37] = 0.33333333333333331*y[1]*y[24] + y[1]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[1]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) - 0.33333333333333331*y[0]*y[31] + y[0]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) + y[0]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333326*y[3]*y[25] - 0.28867513459481281*y[3]*y[33] - 0.16666666666666652*y[3]*y[26] - 0.33333333333333326*y[2]*y[32] + 0.16666666666666652*y[2]*y[33] - 0.28867513459481281*y[2]*y[26] + 0.28867513459481281*y[5]*y[32] - 0.16666666666666652*y[5]*y[25] + 0.33333333333333326*y[5]*y[26] + 0.16666666666666652*y[4]*y[32] + 0.28867513459481281*y[4]*y[25] - 0.33333333333333326*y[4]*y[33] - 1.0*y[31]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[31]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[24]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[24]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) - y[37]; 
data[38] = -y[38] + atan2(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), -p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0])); 
data[39] = -y[39] + atan2(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), -p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953)); 
data[40] = -y[40] + atan2(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), -p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905)); 
data[41] = -y[41] + 3.4641016151377544*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2))/u[12]; 
data[42] = -y[42] + 3.4641016151377544*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))/u[12]; 
data[43] = -y[43] + 3.4641016151377544*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))/u[12]; 
data[44] = p[18]*(-x[2] + y[45] + x[1]/p[19]) - y[44] + 1; 
data[45] = p[22]*x[8] + x[3] - y[45] - (y[44] - u[19])/p[17]; 
data[46] = y[44] - y[46]; 
data[47] = p[25]*x[8] + p[24]*(1 - y[46]) - y[47]; 

}

void h_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14]; 
data[8] = sqrt(pow(y[9], 2) + pow(y[8], 2)); 
data[9] = sqrt(pow(y[11], 2) + pow(y[10], 2)); 
data[10] = sqrt(pow(y[13], 2) + pow(y[12], 2)); 
data[11] = sqrt(1.0*pow(y[31], 2) + pow(y[24], 2)); 
data[12] = sqrt(1.0*pow(y[32], 2) + pow(y[25], 2)); 
data[13] = sqrt(1.0*pow(y[33], 2) + pow(y[26], 2)); 
data[14] = sqrt(1.0*pow(y[34], 2) + pow(y[27], 2)); 
data[15] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666657*y[32] - 0.28867513459481287*y[25]) - y[1]*(0.16666666666666677*y[33] + 0.28867513459481275*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(-0.28867513459481287*y[32] - 0.16666666666666657*y[25]) + y[0]*(0.28867513459481275*y[33] - 0.16666666666666677*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] + 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] - 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] - 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] + 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[31]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[24]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[24]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]); 
data[16] = 0.33333333333333331*y[1]*y[31] - y[1]*(0.16666666666666677*y[32] + 0.28867513459481275*y[25]) - y[1]*(0.16666666666666657*y[33] - 0.28867513459481287*y[26]) + 0.33333333333333331*y[0]*y[24] + y[0]*(0.28867513459481275*y[32] - 0.16666666666666677*y[25]) + y[0]*(-0.28867513459481287*y[33] - 0.16666666666666657*y[26]) + 0.33333333333333326*y[3]*y[32] - 0.16666666666666652*y[3]*y[33] - 0.28867513459481281*y[3]*y[26] + 0.33333333333333326*y[2]*y[25] + 0.28867513459481281*y[2]*y[33] - 0.16666666666666652*y[2]*y[26] - 0.16666666666666652*y[5]*y[32] + 0.28867513459481281*y[5]*y[25] + 0.33333333333333326*y[5]*y[33] - 0.28867513459481281*y[4]*y[32] - 0.16666666666666652*y[4]*y[25] + 0.33333333333333326*y[4]*y[26] + 1.0*y[31]*(-0.16666666666666677*y[3] - 0.28867513459481275*y[2]) + 1.0*y[31]*(-0.16666666666666657*y[5] + 0.28867513459481287*y[4]) + 1.0*y[24]*(0.28867513459481275*y[3] - 0.16666666666666677*y[2]) + 1.0*y[24]*(-0.28867513459481287*y[5] - 0.16666666666666657*y[4]); 
data[17] = 0.33333333333333331*y[1]*y[31] + 0.33333333333333331*y[1]*y[32] + 0.33333333333333331*y[1]*y[33] + 0.33333333333333331*y[0]*y[24] + 0.33333333333333331*y[0]*y[25] + 0.33333333333333331*y[0]*y[26] + 0.33333333333333331*y[3]*y[31] + 0.33333333333333331*y[3]*y[32] + 0.33333333333333331*y[3]*y[33] + 0.33333333333333331*y[2]*y[24] + 0.33333333333333331*y[2]*y[25] + 0.33333333333333331*y[2]*y[26] + 0.33333333333333331*y[5]*y[31] + 0.33333333333333331*y[5]*y[32] + 0.33333333333333331*y[5]*y[33] + 0.33333333333333331*y[4]*y[24] + 0.33333333333333331*y[4]*y[25] + 0.33333333333333331*y[4]*y[26]; 
data[18] = y[41]; 
data[19] = y[42]; 
data[20] = y[43]; 
data[21] = u[13]; 
data[22] = u[14]; 
data[23] = y[29]; 
data[24] = 1.0*y[1]*y[31] + y[0]*y[24] + 1.0*y[3]*y[32] + y[2]*y[25] + 1.0*y[5]*y[33] + y[4]*y[26] + 1.0*y[7]*y[33] + y[6]*y[26]; 
data[25] = u[22]; 
data[26] = u[23]; 
data[27] = u[24]; 
data[28] = u[26]; 
data[29] = u[27]; 
data[30] = u[28]; 
data[31] = u[20]; 
data[32] = u[19]; 
data[33] = x[0]; 
data[34] = 1.4142135623730951*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2)); 
data[35] = 1.4142135623730951*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2)); 
data[36] = 1.4142135623730951*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2)); 
data[37] = pow(p[13], 2)/p[7]; 
data[38] = x[8]; 
data[39] = u[30]; 

}

void de_jac_ini_up_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[23]; 
data[116] = -1/p[14]; 
data[159] = 1/(p[7]*p[14]); 
data[174] = -1/p[15]; 
data[232] = -1/p[16]; 
data[290] = -1/p[16]; 
data[348] = -1/p[16]; 
data[406] = -1/p[16]; 
data[464] = -p[26]; 
data[1914] = -p[2]; 
data[1921] = 1.0*p[1]; 
data[1972] = -p[2]; 
data[1979] = 1.0*p[1]; 
data[2030] = -p[2]; 
data[2037] = 1.0*p[1]; 
data[2088] = -p[4]; 
data[2095] = 1.0*p[3]; 
data[2146] = p[6]/(pow(p[6], 2) + 1.0*pow(p[5], 2)); 
data[2153] = 1.0*p[5]/(pow(p[6], 2) + 1.0*pow(p[5], 2)); 
data[2199] = -1.0*p[1]; 
data[2206] = -1.0*p[2]; 
data[2257] = -1.0*p[1]; 
data[2264] = -1.0*p[2]; 
data[2315] = -1.0*p[1]; 
data[2322] = -1.0*p[2]; 
data[2373] = -1.0*p[3]; 
data[2380] = -1.0*p[4]; 
data[2431] = -1.0*p[5]/(pow(p[6], 2) + 1.0*pow(p[5], 2)); 
data[2438] = 1.0*p[6]/(pow(p[6], 2) + 1.0*pow(p[5], 2)); 
data[2546] = 1.0/u[12]; 
data[3022] = p[18]/p[19]; 
data[3023] = -p[18]; 
data[3075] = p[18]; 
data[3086] = p[22]; 
data[3131] = -1/p[17]; 
data[3200] = p[25]; 
data[3247] = -p[24]; 

}

void de_jac_ini_xy_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[1442] = p[0]*(2*y[8]*u[2] - 2*y[14]*u[2] + y[16]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[17] + y[8]*y[16] - y[15]*y[17] - y[14]*y[16])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1443] = p[0]*(2*y[9]*u[2] - 2*y[15]*u[2] + y[17]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[17] + y[8]*y[16] - y[15]*y[17] - y[14]*y[16])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1448] = p[0]*(-2*y[8]*u[2] + 2*y[14]*u[2] - y[16]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1449] = p[0]*(-2*y[9]*u[2] + 2*y[15]*u[2] - y[17]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1450] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1451] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1501] = p[0]*(2*y[10]*u[6] - 2*y[14]*u[6] + y[18]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[19] + y[10]*y[18] - y[15]*y[19] - y[14]*y[18])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1502] = p[0]*(2*y[11]*u[6] - 2*y[15]*u[6] + y[19]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[19] + y[10]*y[18] - y[15]*y[19] - y[14]*y[18])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1505] = p[0]*(-2*y[10]*u[6] + 2*y[14]*u[6] - y[18]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1506] = p[0]*(-2*y[11]*u[6] + 2*y[15]*u[6] - y[19]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1509] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1510] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1560] = p[0]*(2*y[12]*u[10] - 2*y[14]*u[10] + y[20]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[21] + y[12]*y[20] - y[15]*y[21] - y[14]*y[20])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1561] = p[0]*(2*y[13]*u[10] - 2*y[15]*u[10] + y[21]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[21] + y[12]*y[20] - y[15]*y[21] - y[14]*y[20])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1562] = p[0]*(-2*y[12]*u[10] + 2*y[14]*u[10] - y[20]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1563] = p[0]*(-2*y[13]*u[10] + 2*y[15]*u[10] - y[21]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1568] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1569] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1613] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[17]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[16] - y[8]*y[17] - y[15]*y[16] + y[14]*y[17])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1614] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[16]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[16] - y[8]*y[17] - y[15]*y[16] + y[14]*y[17])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1619] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[17]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1620] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[16]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1621] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1622] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1672] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[19]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[18] - y[10]*y[19] - y[15]*y[18] + y[14]*y[19])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1673] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[18]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[18] - y[10]*y[19] - y[15]*y[18] + y[14]*y[19])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1676] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[19]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1677] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[18]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1680] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1681] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1731] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[21]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[20] - y[12]*y[21] - y[15]*y[20] + y[14]*y[21])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1732] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[20]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[20] - y[12]*y[21] - y[15]*y[20] + y[14]*y[21])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
))); 
data[1733] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[21]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1734] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[20]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
))); 
data[1739] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1740] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)); 
data[1928] = -0.40824829046386307*y[41]*u[12]*sin(u[14] + y[38]); 
data[1931] = 0.40824829046386307*u[12]*cos(u[14] + y[38]); 
data[1986] = -0.40824829046386307*y[42]*u[12]*sin(u[14] + y[39]); 
data[1989] = 0.40824829046386307*u[12]*cos(u[14] + y[39]); 
data[2044] = -0.40824829046386307*y[43]*u[12]*sin(u[14] + y[40]); 
data[2047] = 0.40824829046386307*u[12]*cos(u[14] + y[40]); 
data[2213] = 0.40824829046386307*y[41]*u[12]*cos(u[14] + y[38]); 
data[2216] = 0.40824829046386307*u[12]*sin(u[14] + y[38]); 
data[2271] = 0.40824829046386307*y[42]*u[12]*cos(u[14] + y[39]); 
data[2274] = 0.40824829046386307*u[12]*sin(u[14] + y[39]); 
data[2329] = 0.40824829046386307*y[43]*u[12]*cos(u[14] + y[40]); 
data[2332] = 0.40824829046386307*u[12]*sin(u[14] + y[40]); 
data[2484] = 2*p[8]*y[24] + p[9]*y[24]/sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[41]*u[12]*cos(u[14] + y[38]); 
data[2485] = 2*p[8]*y[25] + p[9]*y[25]/sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[42]*u[12]*cos(u[14] + y[39]); 
data[2486] = 2*p[8]*y[26] + p[9]*y[26]/sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[43]*u[12]*cos(u[14] + y[40]) + 0.40824829046386307*u[13]*u[12]*cos(u[15]); 
data[2487] = 2*p[8]*y[27] + p[9]*y[27]/sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7); 
data[2491] = 2*p[8]*y[31] + p[9]*y[31]/sqrt(pow(y[31], 2) + pow(y[24], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[41]*u[12]*sin(u[14] + y[38]); 
data[2492] = 2*p[8]*y[32] + p[9]*y[32]/sqrt(pow(y[32], 2) + pow(y[25], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[42]*u[12]*sin(u[14] + y[39]); 
data[2493] = 2*p[8]*y[33] + p[9]*y[33]/sqrt(pow(y[33], 2) + pow(y[26], 2) + 9.9999999999999995e-7) + 0.40824829046386307*y[43]*u[12]*sin(u[14] + y[40]) + 0.40824829046386307*u[13]*u[12]*sin(u[15]); 
data[2494] = 2*p[8]*y[34] + p[9]*y[34]/sqrt(pow(y[34], 2) + pow(y[27], 2) + 9.9999999999999995e-7); 
data[2498] = 0.40824829046386307*y[31]*y[41]*u[12]*cos(u[14] + y[38]) - 0.40824829046386307*y[24]*y[41]*u[12]*sin(u[14] + y[38]); 
data[2499] = 0.40824829046386307*y[32]*y[42]*u[12]*cos(u[14] + y[39]) - 0.40824829046386307*y[25]*y[42]*u[12]*sin(u[14] + y[39]); 
data[2500] = 0.40824829046386307*y[33]*y[43]*u[12]*cos(u[14] + y[40]) - 0.40824829046386307*y[26]*y[43]*u[12]*sin(u[14] + y[40]); 
data[2501] = 0.40824829046386307*y[31]*u[12]*sin(u[14] + y[38]) + 0.40824829046386307*y[24]*u[12]*cos(u[14] + y[38]); 
data[2502] = 0.40824829046386307*y[32]*u[12]*sin(u[14] + y[39]) + 0.40824829046386307*y[25]*u[12]*cos(u[14] + y[39]); 
data[2503] = 0.40824829046386307*y[33]*u[12]*sin(u[14] + y[40]) + 0.40824829046386307*y[26]*u[12]*cos(u[14] + y[40]); 
data[2574] = 0.33333333333333331*y[24] - 0.28867513459481287*y[32] - 0.16666666666666657*y[25] + 0.28867513459481275*y[33] - 0.16666666666666677*y[26]; 
data[2575] = 0.33333333333333331*y[31] - 0.16666666666666657*y[32] + 0.28867513459481287*y[25] - 0.16666666666666677*y[33] - 0.28867513459481275*y[26]; 
data[2576] = 0.28867513459481287*y[31] - 0.16666666666666657*y[24] + 0.33333333333333326*y[25] - 0.28867513459481281*y[33] - 0.16666666666666652*y[26]; 
data[2577] = -0.16666666666666657*y[31] - 0.28867513459481287*y[24] + 0.33333333333333326*y[32] - 0.16666666666666652*y[33] + 0.28867513459481281*y[26]; 
data[2578] = -0.28867513459481275*y[31] - 0.16666666666666677*y[24] + 0.28867513459481281*y[32] - 0.16666666666666652*y[25] + 0.33333333333333326*y[26]; 
data[2579] = -0.16666666666666677*y[31] + 0.28867513459481275*y[24] - 0.16666666666666652*y[32] - 0.28867513459481281*y[25] + 0.33333333333333326*y[33]; 
data[2598] = 0.33333333333333331*y[0] - 0.28867513459481287*y[3] - 0.16666666666666657*y[2] + 0.28867513459481275*y[5] - 0.16666666666666677*y[4]; 
data[2599] = 0.28867513459481287*y[1] - 0.16666666666666657*y[0] + 0.33333333333333326*y[2] - 0.28867513459481281*y[5] - 0.16666666666666652*y[4]; 
data[2600] = -0.28867513459481275*y[1] - 0.16666666666666677*y[0] + 0.28867513459481281*y[3] - 0.16666666666666652*y[2] + 0.33333333333333326*y[4]; 
data[2605] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[2606] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[2607] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[2631] = -0.33333333333333331*y[31] + 0.16666666666666657*y[32] - 0.28867513459481287*y[25] + 0.16666666666666677*y[33] + 0.28867513459481275*y[26]; 
data[2632] = 0.33333333333333331*y[24] - 0.28867513459481287*y[32] - 0.16666666666666657*y[25] + 0.28867513459481275*y[33] - 0.16666666666666677*y[26]; 
data[2633] = 0.16666666666666657*y[31] + 0.28867513459481287*y[24] - 0.33333333333333326*y[32] + 0.16666666666666652*y[33] - 0.28867513459481281*y[26]; 
data[2634] = 0.28867513459481287*y[31] - 0.16666666666666657*y[24] + 0.33333333333333326*y[25] - 0.28867513459481281*y[33] - 0.16666666666666652*y[26]; 
data[2635] = 0.16666666666666677*y[31] - 0.28867513459481275*y[24] + 0.16666666666666652*y[32] + 0.28867513459481281*y[25] - 0.33333333333333326*y[33]; 
data[2636] = -0.28867513459481275*y[31] - 0.16666666666666677*y[24] + 0.28867513459481281*y[32] - 0.16666666666666652*y[25] + 0.33333333333333326*y[26]; 
data[2655] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[2656] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[2657] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[2662] = -0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4]; 
data[2663] = -0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4]; 
data[2664] = 0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4]; 
data[2679] = -(x[4] + u[22])*(p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) - 1.0*(x[4] + u[22])*sin(u[16] + x[0]))*sin(u[16] + x[0])/(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2)) + 1.0*(x[4] + u[22])*(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]))*cos(u[16] + x[0])/(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2)); 
data[2683] = (p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) - 1.0*(x[4] + u[22])*sin(u[16] + x[0]))*cos(u[16] + x[0])/(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2)) + 1.0*(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]))*sin(u[16] + x[0])/(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2)); 
data[2712] = -p[12]*p[7]*(p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) - 1.0*(x[4] + u[22])*sin(u[16] + x[0]))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2))) - 1.0*p[7]*p[11]*(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2))); 
data[2719] = -1.0*p[12]*p[7]*(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2))) + 1.0*p[7]*p[11]*(p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) - 1.0*(x[4] + u[22])*sin(u[16] + x[0]))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[31] + 1.0*p[11]*y[24])/pow(p[13], 2) + 1.0*(x[4] + u[22])*sin(u[16] + x[0]), 2) + pow(-p[7]*(p[12]*y[24] - 1.0*p[11]*y[31])/pow(p[13], 2) + (x[4] + u[22])*cos(u[16] + x[0]), 2))); 
data[2736] = -(x[5] + u[23])*(p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) - 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953))*sin(u[17] + x[0] - 2.0943951023931953)/(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2)) + 1.0*(x[5] + u[23])*(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953))*cos(u[17] + x[0] - 2.0943951023931953)/(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2)); 
data[2741] = (p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) - 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953))*cos(u[17] + x[0] - 2.0943951023931953)/(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2)) + 1.0*(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953))*sin(u[17] + x[0] - 2.0943951023931953)/(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2)); 
data[2770] = -p[12]*p[7]*(p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) - 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2))) - 1.0*p[7]*p[11]*(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2777] = -1.0*p[12]*p[7]*(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2))) + 1.0*p[7]*p[11]*(p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) - 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[32] + 1.0*p[11]*y[25])/pow(p[13], 2) + 1.0*(x[5] + u[23])*sin(u[17] + x[0] - 2.0943951023931953), 2) + pow(-p[7]*(p[12]*y[25] - 1.0*p[11]*y[32])/pow(p[13], 2) + (x[5] + u[23])*cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2793] = -(x[6] + u[24])*(p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) - 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905))*sin(u[18] + x[0] - 4.1887902047863905)/(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2)) + 1.0*(x[6] + u[24])*(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905))*cos(u[18] + x[0] - 4.1887902047863905)/(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2)); 
data[2799] = (p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) - 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905))*cos(u[18] + x[0] - 4.1887902047863905)/(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2)) + 1.0*(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905))*sin(u[18] + x[0] - 4.1887902047863905)/(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2)); 
data[2828] = -p[12]*p[7]*(p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) - 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2))) - 1.0*p[7]*p[11]*(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2))); 
data[2835] = -1.0*p[12]*p[7]*(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2))) + 1.0*p[7]*p[11]*(p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) - 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905))/(pow(p[13], 2)*(pow(-p[7]*(1.0*p[12]*y[33] + 1.0*p[11]*y[26])/pow(p[13], 2) + 1.0*(x[6] + u[24])*sin(u[18] + x[0] - 4.1887902047863905), 2) + pow(-p[7]*(p[12]*y[26] - 1.0*p[11]*y[33])/pow(p[13], 2) + (x[6] + u[24])*cos(u[18] + x[0] - 4.1887902047863905), 2))); 
data[2850] = 3.4641016151377544*(-1.0/2.0*x[4]*p[12]*p[7]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) + (1.0/2.0)*x[4]*p[12]*p[7]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*x[4]*p[7]*p[11]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*x[4]*p[7]*p[11]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) + (1.0/2.0)*p[12]*p[7]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2))); 
data[2854] = 3.4641016151377544*(0.5*x[4]*pow(sin(u[16] + x[0]), 2) + 0.5*x[4]*pow(cos(u[16] + x[0]), 2) - 1.0/2.0*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + (1.0/2.0)*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + (1.0/2.0)*u[22]*pow(sin(u[16] + x[0]), 2) + (1.0/2.0)*u[22]*pow(cos(u[16] + x[0]), 2))/(u[12]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2))); 
data[2883] = 3.4641016151377544*(-1.0/2.0*x[4]*p[12]*p[7]*cos(u[16] + x[0])/pow(p[13], 2) - 1.0/2.0*x[4]*p[7]*p[11]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[24]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[22]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[24]/pow(p[13], 4) - 1.0/2.0*p[7]*p[11]*u[22]*sin(u[16] + x[0])/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2))); 
data[2890] = 3.4641016151377544*(-1.0/2.0*x[4]*p[12]*p[7]*sin(u[16] + x[0])/pow(p[13], 2) + (1.0/2.0)*x[4]*p[7]*p[11]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[31]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[22]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[31]/pow(p[13], 4) + (1.0/2.0)*p[7]*p[11]*u[22]*cos(u[16] + x[0])/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[4], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(x[4], 2)*pow(cos(u[16] + x[0]), 2) - x[4]*p[12]*p[7]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - x[4]*p[12]*p[7]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + x[4]*p[7]*p[11]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - x[4]*p[7]*p[11]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + x[4]*u[22]*pow(sin(u[16] + x[0]), 2) + x[4]*u[22]*pow(cos(u[16] + x[0]), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[24], 2)/pow(p[13], 4) - p[12]*p[7]*u[22]*y[31]*sin(u[16] + x[0])/pow(p[13], 2) - p[12]*p[7]*u[22]*y[24]*cos(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[31], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[24], 2)/pow(p[13], 4) + p[7]*p[11]*u[22]*y[31]*cos(u[16] + x[0])/pow(p[13], 2) - p[7]*p[11]*u[22]*y[24]*sin(u[16] + x[0])/pow(p[13], 2) + 0.5*pow(u[22], 2)*pow(sin(u[16] + x[0]), 2) + 0.5*pow(u[22], 2)*pow(cos(u[16] + x[0]), 2))); 
data[2907] = 3.4641016151377544*(-1.0/2.0*x[5]*p[12]*p[7]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + (1.0/2.0)*x[5]*p[12]*p[7]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*x[5]*p[7]*p[11]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*x[5]*p[7]*p[11]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + (1.0/2.0)*p[12]*p[7]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2912] = 3.4641016151377544*(0.5*x[5]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*x[5]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - 1.0/2.0*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + (1.0/2.0)*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + (1.0/2.0)*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + (1.0/2.0)*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))/(u[12]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2941] = 3.4641016151377544*(-1.0/2.0*x[5]*p[12]*p[7]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - 1.0/2.0*x[5]*p[7]*p[11]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[25]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[23]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[25]/pow(p[13], 4) - 1.0/2.0*p[7]*p[11]*u[23]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2948] = 3.4641016151377544*(-1.0/2.0*x[5]*p[12]*p[7]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + (1.0/2.0)*x[5]*p[7]*p[11]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[32]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[23]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[32]/pow(p[13], 4) + (1.0/2.0)*p[7]*p[11]*u[23]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[5], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(x[5], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) - x[5]*p[12]*p[7]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[12]*p[7]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*p[7]*p[11]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - x[5]*p[7]*p[11]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + x[5]*u[23]*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + x[5]*u[23]*pow(cos(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[25], 2)/pow(p[13], 4) - p[12]*p[7]*u[23]*y[32]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[12]*p[7]*u[23]*y[25]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[32], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[25], 2)/pow(p[13], 4) + p[7]*p[11]*u[23]*y[32]*cos(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) - p[7]*p[11]*u[23]*y[25]*sin(u[17] + x[0] - 2.0943951023931953)/pow(p[13], 2) + 0.5*pow(u[23], 2)*pow(sin(u[17] + x[0] - 2.0943951023931953), 2) + 0.5*pow(u[23], 2)*pow(cos(u[17] + x[0] - 2.0943951023931953), 2))); 
data[2964] = 3.4641016151377544*(-1.0/2.0*x[6]*p[12]*p[7]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + (1.0/2.0)*x[6]*p[12]*p[7]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*x[6]*p[7]*p[11]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*x[6]*p[7]*p[11]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + (1.0/2.0)*p[12]*p[7]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))); 
data[2970] = 3.4641016151377544*(0.5*x[6]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*x[6]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - 1.0/2.0*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + (1.0/2.0)*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + (1.0/2.0)*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + (1.0/2.0)*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))/(u[12]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))); 
data[2999] = 3.4641016151377544*(-1.0/2.0*x[6]*p[12]*p[7]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - 1.0/2.0*x[6]*p[7]*p[11]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[26]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[24]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[26]/pow(p[13], 4) - 1.0/2.0*p[7]*p[11]*u[24]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))); 
data[3006] = 3.4641016151377544*(-1.0/2.0*x[6]*p[12]*p[7]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + (1.0/2.0)*x[6]*p[7]*p[11]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*y[33]/pow(p[13], 4) - 1.0/2.0*p[12]*p[7]*u[24]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*y[33]/pow(p[13], 4) + (1.0/2.0)*p[7]*p[11]*u[24]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2))/(u[12]*sqrt(0.5*pow(x[6], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(x[6], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) - x[6]*p[12]*p[7]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[12]*p[7]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*p[7]*p[11]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - x[6]*p[7]*p[11]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + x[6]*u[24]*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + x[6]*u[24]*pow(cos(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[12], 2)*pow(p[7], 2)*pow(y[26], 2)/pow(p[13], 4) - p[12]*p[7]*u[24]*y[33]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[12]*p[7]*u[24]*y[26]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[33], 2)/pow(p[13], 4) + 0.5*pow(p[7], 2)*pow(p[11], 2)*pow(y[26], 2)/pow(p[13], 4) + p[7]*p[11]*u[24]*y[33]*cos(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) - p[7]*p[11]*u[24]*y[26]*sin(u[18] + x[0] - 4.1887902047863905)/pow(p[13], 2) + 0.5*pow(u[24], 2)*pow(sin(u[18] + x[0] - 4.1887902047863905), 2) + 0.5*pow(u[24], 2)*pow(cos(u[18] + x[0] - 4.1887902047863905), 2))); 

}

void de_jac_ini_num_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[53] = 314.15926535897933; 
data[55] = -314.15926535897933; 
data[59] = -1; 
data[111] = 1; 
data[511] = -1; 
data[522] = 184.75059629806532; 
data[523] = 94.615102344455565; 
data[524] = -69.368640085185334; 
data[525] = -17.974903844364775; 
data[526] = -43.675799641021136; 
data[527] = -40.74868784166955; 
data[528] = -69.368640085185348; 
data[529] = -17.974903844364817; 
data[530] = -184.75059629806532; 
data[531] = -94.615102344455565; 
data[532] = 69.368640085185334; 
data[533] = 17.974903844364775; 
data[534] = 43.675799641021136; 
data[535] = 40.74868784166955; 
data[536] = 69.368640085185348; 
data[537] = 17.974903844364817; 
data[546] = -1; 
data[579] = -94.615102344455565; 
data[580] = 184.75059629806532; 
data[581] = 17.974903844364775; 
data[582] = -69.368640085185334; 
data[583] = 40.74868784166955; 
data[584] = -43.675799641021136; 
data[585] = 17.974903844364817; 
data[586] = -69.368640085185348; 
data[587] = 94.615102344455565; 
data[588] = -184.75059629806532; 
data[589] = -17.974903844364775; 
data[590] = 69.368640085185334; 
data[591] = -40.74868784166955; 
data[592] = 43.675799641021136; 
data[593] = -17.974903844364817; 
data[594] = 69.368640085185348; 
data[610] = -1; 
data[636] = -69.368640085185319; 
data[637] = -17.974903844364796; 
data[638] = 184.75059629806523; 
data[639] = 94.615102344455607; 
data[640] = -69.368640085185291; 
data[641] = -17.974903844364814; 
data[642] = -43.675799641021079; 
data[643] = -40.748687841669586; 
data[644] = 69.368640085185319; 
data[645] = 17.974903844364796; 
data[646] = -184.75059629806523; 
data[647] = -94.615102344455607; 
data[648] = 69.368640085185291; 
data[649] = 17.974903844364814; 
data[650] = 43.675799641021079; 
data[651] = 40.748687841669586; 
data[661] = -1; 
data[693] = 17.974903844364796; 
data[694] = -69.368640085185319; 
data[695] = -94.615102344455607; 
data[696] = 184.75059629806523; 
data[697] = 17.974903844364814; 
data[698] = -69.368640085185291; 
data[699] = 40.748687841669586; 
data[700] = -43.675799641021079; 
data[701] = -17.974903844364796; 
data[702] = 69.368640085185319; 
data[703] = 94.615102344455607; 
data[704] = -184.75059629806523; 
data[705] = -17.974903844364814; 
data[706] = 69.368640085185291; 
data[707] = -40.748687841669586; 
data[708] = 43.675799641021079; 
data[725] = -1; 
data[750] = -43.675799641021122; 
data[751] = -40.748687841669557; 
data[752] = -69.368640085185277; 
data[753] = -17.97490384436481; 
data[754] = 184.75059629806526; 
data[755] = 94.615102344455636; 
data[756] = -69.368640085185277; 
data[757] = -17.974903844364817; 
data[758] = 43.675799641021122; 
data[759] = 40.748687841669557; 
data[760] = 69.368640085185277; 
data[761] = 17.97490384436481; 
data[762] = -184.75059629806526; 
data[763] = -94.615102344455636; 
data[764] = 69.368640085185277; 
data[765] = 17.974903844364817; 
data[776] = -1; 
data[807] = 40.748687841669557; 
data[808] = -43.675799641021122; 
data[809] = 17.97490384436481; 
data[810] = -69.368640085185277; 
data[811] = -94.615102344455636; 
data[812] = 184.75059629806526; 
data[813] = 17.974903844364817; 
data[814] = -69.368640085185277; 
data[815] = -40.748687841669557; 
data[816] = 43.675799641021122; 
data[817] = -17.97490384436481; 
data[818] = 69.368640085185277; 
data[819] = 94.615102344455636; 
data[820] = -184.75059629806526; 
data[821] = -17.974903844364817; 
data[822] = 69.368640085185277; 
data[840] = -1; 
data[864] = -69.368640085185334; 
data[865] = -17.9749038443648; 
data[866] = -43.675799641021086; 
data[867] = -40.748687841669586; 
data[868] = -69.368640085185291; 
data[869] = -17.974903844364821; 
data[870] = 184.75059629806523; 
data[871] = 94.61510234445565; 
data[872] = 69.368640085185334; 
data[873] = 17.9749038443648; 
data[874] = 43.675799641021086; 
data[875] = 40.748687841669586; 
data[876] = 69.368640085185291; 
data[877] = 17.974903844364821; 
data[878] = -184.75059629806523; 
data[879] = -94.61510234445565; 
data[891] = -1; 
data[921] = 17.9749038443648; 
data[922] = -69.368640085185334; 
data[923] = 40.748687841669586; 
data[924] = -43.675799641021086; 
data[925] = 17.974903844364821; 
data[926] = -69.368640085185291; 
data[927] = -94.61510234445565; 
data[928] = 184.75059629806523; 
data[929] = -17.9749038443648; 
data[930] = 69.368640085185334; 
data[931] = -40.748687841669586; 
data[932] = 43.675799641021086; 
data[933] = -17.974903844364821; 
data[934] = 69.368640085185291; 
data[935] = 94.61510234445565; 
data[936] = -184.75059629806523; 
data[955] = -1; 
data[978] = -184.75059629806532; 
data[979] = -94.615102344455565; 
data[980] = 69.368640085185334; 
data[981] = 17.974903844364775; 
data[982] = 43.675799641021136; 
data[983] = 40.74868784166955; 
data[984] = 69.368640085185348; 
data[985] = 17.974903844364817; 
data[986] = 184.75059629806532; 
data[987] = 94.615102344455565; 
data[988] = -69.368640085185334; 
data[989] = -17.974903844364775; 
data[990] = -43.675799641021136; 
data[991] = -40.74868784166955; 
data[992] = -69.368640085185348; 
data[993] = -17.974903844364817; 
data[994] = -1; 
data[1035] = 94.615102344455565; 
data[1036] = -184.75059629806532; 
data[1037] = -17.974903844364775; 
data[1038] = 69.368640085185334; 
data[1039] = -40.74868784166955; 
data[1040] = 43.675799641021136; 
data[1041] = -17.974903844364817; 
data[1042] = 69.368640085185348; 
data[1043] = -94.615102344455565; 
data[1044] = 184.75059629806532; 
data[1045] = 17.974903844364775; 
data[1046] = -69.368640085185334; 
data[1047] = 40.74868784166955; 
data[1048] = -43.675799641021136; 
data[1049] = 17.974903844364817; 
data[1050] = -69.368640085185348; 
data[1052] = -1; 
data[1092] = 69.368640085185319; 
data[1093] = 17.974903844364796; 
data[1094] = -184.75059629806523; 
data[1095] = -94.615102344455607; 
data[1096] = 69.368640085185291; 
data[1097] = 17.974903844364814; 
data[1098] = 43.675799641021079; 
data[1099] = 40.748687841669586; 
data[1100] = -69.368640085185319; 
data[1101] = -17.974903844364796; 
data[1102] = 184.75059629806523; 
data[1103] = 94.615102344455607; 
data[1104] = -69.368640085185291; 
data[1105] = -17.974903844364814; 
data[1106] = -43.675799641021079; 
data[1107] = -40.748687841669586; 
data[1110] = -1; 
data[1149] = -17.974903844364796; 
data[1150] = 69.368640085185319; 
data[1151] = 94.615102344455607; 
data[1152] = -184.75059629806523; 
data[1153] = -17.974903844364814; 
data[1154] = 69.368640085185291; 
data[1155] = -40.748687841669586; 
data[1156] = 43.675799641021079; 
data[1157] = 17.974903844364796; 
data[1158] = -69.368640085185319; 
data[1159] = -94.615102344455607; 
data[1160] = 184.75059629806523; 
data[1161] = 17.974903844364814; 
data[1162] = -69.368640085185291; 
data[1163] = 40.748687841669586; 
data[1164] = -43.675799641021079; 
data[1168] = -1; 
data[1206] = 43.675799641021122; 
data[1207] = 40.748687841669557; 
data[1208] = 69.368640085185277; 
data[1209] = 17.97490384436481; 
data[1210] = -184.75059629806526; 
data[1211] = -94.615102344455636; 
data[1212] = 69.368640085185277; 
data[1213] = 17.974903844364817; 
data[1214] = -43.675799641021122; 
data[1215] = -40.748687841669557; 
data[1216] = -69.368640085185277; 
data[1217] = -17.97490384436481; 
data[1218] = 184.75059629806526; 
data[1219] = 94.615102344455636; 
data[1220] = -69.368640085185277; 
data[1221] = -17.974903844364817; 
data[1226] = -1; 
data[1263] = -40.748687841669557; 
data[1264] = 43.675799641021122; 
data[1265] = -17.97490384436481; 
data[1266] = 69.368640085185277; 
data[1267] = 94.615102344455636; 
data[1268] = -184.75059629806526; 
data[1269] = -17.974903844364817; 
data[1270] = 69.368640085185277; 
data[1271] = 40.748687841669557; 
data[1272] = -43.675799641021122; 
data[1273] = 17.97490384436481; 
data[1274] = -69.368640085185277; 
data[1275] = -94.615102344455636; 
data[1276] = 184.75059629806526; 
data[1277] = 17.974903844364817; 
data[1278] = -69.368640085185277; 
data[1284] = -1; 
data[1320] = 69.368640085185334; 
data[1321] = 17.9749038443648; 
data[1322] = 43.675799641021086; 
data[1323] = 40.748687841669586; 
data[1324] = 69.368640085185291; 
data[1325] = 17.974903844364821; 
data[1326] = -184.75059629806523; 
data[1327] = -94.61510234445565; 
data[1328] = -69.368640085185334; 
data[1329] = -17.9749038443648; 
data[1330] = -43.675799641021086; 
data[1331] = -40.748687841669586; 
data[1332] = -69.368640085185291; 
data[1333] = -17.974903844364821; 
data[1334] = 184.75059629806523; 
data[1335] = 94.61510234445565; 
data[1342] = -1; 
data[1377] = -17.9749038443648; 
data[1378] = 69.368640085185334; 
data[1379] = -40.748687841669586; 
data[1380] = 43.675799641021086; 
data[1381] = -17.974903844364821; 
data[1382] = 69.368640085185291; 
data[1383] = 94.61510234445565; 
data[1384] = -184.75059629806523; 
data[1385] = 17.9749038443648; 
data[1386] = -69.368640085185334; 
data[1387] = 40.748687841669586; 
data[1388] = -43.675799641021086; 
data[1389] = 17.974903844364821; 
data[1390] = -69.368640085185291; 
data[1391] = -94.61510234445565; 
data[1392] = 184.75059629806523; 
data[1400] = -1; 
data[1792] = 1; 
data[1794] = 1; 
data[1796] = 1; 
data[1798] = 1; 
data[1850] = 1; 
data[1852] = 1; 
data[1854] = 1; 
data[1856] = 1; 
data[1890] = -1; 
data[1918] = 1; 
data[1949] = -1; 
data[1975] = 1; 
data[2008] = -1; 
data[2032] = 1; 
data[2067] = -1; 
data[2089] = 1; 
data[2142] = 1; 
data[2143] = 1; 
data[2144] = 1; 
data[2145] = 1; 
data[2176] = -1.0; 
data[2210] = 1.0; 
data[2235] = -1.0; 
data[2267] = 1.0; 
data[2294] = -1.0; 
data[2324] = 1.0; 
data[2353] = -1.0; 
data[2381] = 1.0; 
data[2434] = 1.0; 
data[2435] = 1.0; 
data[2436] = 1.0; 
data[2437] = 1.0; 
data[2489] = -1; 
data[2547] = -1; 
data[2610] = -1; 
data[2668] = -1; 
data[2726] = -1; 
data[2784] = -1; 
data[2842] = -1; 
data[2900] = -1; 
data[2958] = -1; 
data[3016] = -1; 
data[3074] = -1; 
data[3081] = 1; 
data[3132] = -1; 
data[3188] = 1; 
data[3190] = -1; 
data[3248] = -1; 

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_num_eval _cffi_d_de_jac_ini_num_eval
#endif

static void _cffi_d_de_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_up_eval _cffi_d_de_jac_ini_up_eval
#endif

static void _cffi_d_de_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_xy_eval _cffi_d_de_jac_ini_xy_eval
#endif

static void _cffi_d_f_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_ini_eval _cffi_d_f_ini_eval
#endif

static void _cffi_d_g_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_ini_eval _cffi_d_g_ini_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_jac_ini_num_eval", (void *)_cffi_f_de_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_num_eval },
  { "de_jac_ini_up_eval", (void *)_cffi_f_de_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_up_eval },
  { "de_jac_ini_xy_eval", (void *)_cffi_f_de_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_xy_eval },
  { "f_ini_eval", (void *)_cffi_f_f_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_ini_eval },
  { "g_ini_eval", (void *)_cffi_f_g_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_ini_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  6,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_ini_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_ini_cffi(void) { return NULL; }
#  else
     inittemp_ini_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_ini_cffi(void)
{
  return _cffi_init("temp_ini_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_ini_cffi(void)
{
  _cffi_init("temp_ini_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
